@@ -3,13 +3,19 @@ import dash
 import pandas as pd
 from webviz_config.common_cache import CACHE
 from webviz_config.themes import default_theme
-from webviz_config.plugins import ParameterCorrelation
+
+# pylint: disable=no-name-in-module
+from webviz_config.plugins import (
+    ParameterCorrelation,
+)
+
+# pylint: enable=no-name-in-module
 
 # mocked functions
-get_parameters = "webviz_subsurface.plugins._parameter_correlation.get_parameters"
+GET_PARAMETERS = "webviz_subsurface.plugins._parameter_correlation.get_parameters"
 
 
-def test_parameter_corr(dash_duo):
+def test_parameter_corr(dash_duo: dash.testing.composite.DashComposite) -> None:
 
     app = dash.Dash(__name__)
     app.css.config.serve_locally = True
@@ -22,15 +28,17 @@ def test_parameter_corr(dash_duo):
     }
     ensembles = ["iter-0"]
 
-    with mock.patch(get_parameters) as mock_parameters:
+    with mock.patch(GET_PARAMETERS) as mock_parameters:
         mock_parameters.return_value = pd.read_csv("tests/data/parameters.csv")
 
-        p = ParameterCorrelation(app, ensembles)
+        parameter_correlation = ParameterCorrelation(app, ensembles)
 
-        app.layout = p.layout
+        app.layout = parameter_correlation.layout
         dash_duo.start_server(app)
 
-        my_component = dash_duo.find_element(f"#{p.ids('ensemble-all')}")
+        my_component = dash_duo.find_element(
+            f"#{parameter_correlation.ids('ensemble-all')}"
+        )
 
         if not my_component.text.startswith("iter-0"):
             raise AssertionError()
