@@ -9,8 +9,14 @@ from webviz_config.webviz_store import webvizstore
 
 
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
-def scratch_ensemble(ensemble_name: str, ensemble_path: Path):
-    return ScratchEnsemble(ensemble_name, ensemble_path)
+def scratch_ensemble(
+    ensemble_name: str, ensemble_path: Path, filter_file: Union[str, None] = "OK"
+) -> ScratchEnsemble:
+    return (
+        ScratchEnsemble(ensemble_name, ensemble_path)
+        if filter_file is None
+        else ScratchEnsemble(ensemble_name, ensemble_path).filter(filter_file)
+    )
 
 
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
@@ -18,13 +24,11 @@ def load_ensemble_set(
     ensemble_paths: dict,
     ensemble_set_name: str = "EnsembleSet",
     filter_file: Union[str, None] = "OK",
-):
+) -> EnsembleSet:
     return EnsembleSet(
         ensemble_set_name,
         [
-            ScratchEnsemble(ens_name, ens_path)
-            if filter_file is None
-            else ScratchEnsemble(ens_name, ens_path).filter(filter_file)
+            scratch_ensemble(ens_name, ens_path, filter_file)
             for ens_name, ens_path in ensemble_paths.items()
         ],
     )
@@ -107,7 +111,7 @@ def get_realizations(
     return df.sort_values(by=["ENSEMBLE", "REAL"])
 
 
-def find_sens_type(senscase: str):
+def find_sens_type(senscase: str) -> Optional[str]:
     """Finds sensitivity type from sensitivty case.
     If sensitivity case is 'p10_p90', sensitivity type is montecarlo,
     else sensitivity type is set to 'scalar'.
@@ -123,7 +127,9 @@ def find_sens_type(senscase: str):
 
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
 @webvizstore
-def find_surfaces(ensemble_paths: dict, suffix="*.gri", delimiter="--") -> pd.DataFrame:
+def find_surfaces(
+    ensemble_paths: dict, suffix: str = "*.gri", delimiter: str = "--"
+) -> pd.DataFrame:
     """Reads surface file names stored in standard FMU format, and returns a dictionary
     on the following format:
     surface_property:
