@@ -2,13 +2,11 @@ import io
 import base64
 
 import numpy as np
-from matplotlib import cm
 from PIL import Image
 
 
-def array_to_png(
-    tensor, shift=True, colormap=False
-):  # pylint: disable=too-many-branches
+def array_to_png(tensor: np.ndarray, shift: bool = True, colormap: bool = False) -> str:
+    # pylint: disable=too-many-branches
     """The layered map dash component takes in pictures as base64 data
     (or as a link to an existing hosted image). I.e. for containers wanting
     to create pictures on-the-fly from numpy arrays, they have to be converted
@@ -76,9 +74,3 @@ def array_to_png(
     base64_data = base64.b64encode(byte_io.read()).decode("ascii")
 
     return f"data:image/png;base64,{base64_data}"
-
-
-def get_colormap(colormap):
-    return array_to_png(
-        cm.get_cmap(colormap, 256)([np.linspace(0, 1, 256)]), colormap=True
-    )
