@@ -1,6 +1,6 @@
+from typing import List, Optional, Dict, Any, Union, Tuple
 from uuid import uuid4
 import json
-from typing import List, Optional
 
 import pandas as pd
 import dash
@@ -17,41 +17,44 @@ from .._abbreviations.number_formatting import si_prefixed
 class TornadoPlot:
     """### TornadoPlot
 
-This private plugin visualizes a Tornado plot.
-It is meant to be used as a component in other plugin, and is initialized
- with a dataframe of realizations with corresponding sensitivities,
-but without the response values that are to be plotted.
-Instead we registers a dcc.Store which will contain the response values.
+    This private plugin visualizes a Tornado plot.
+    It is meant to be used as a component in other plugin, and is initialized
+     with a dataframe of realizations with corresponding sensitivities,
+    but without the response values that are to be plotted.
+    Instead we registers a dcc.Store which will contain the response values.
 
-To use:
-1. Initialize an instance of this class in a plugin.
-2. Add tornadoplot.layout to the plugin layout
-3. Register a callback that writes a json dump to tornadoplot.storage_id
-The format of the json dump must be ('ENSEMBLE' and 'data' are mandatory, the others optional):
-{'ENSEMBLE': name of ensemble,
- 'data': 2d array of realizations / response values
- 'number_format' (str): Format of the numeric part based on the Python Format Specification
-  Mini-Language e.g. '#.3g' for 3 significant digits, '.2f' for two decimals, or '.0f' for no
-  decimals.
- 'unit' (str): String to append at the end as a unit.
- 'spaced' (bool): Include a space between last numerical digit and SI-prefix.
- 'locked_si_prefix' (str or int): Lock the SI prefix to either a string (e.g. 'm' (milli) or 'M'
-  (mega)), or an integer which is the base 10 exponent (e.g. 3 for kilo, -3 for milli).
-}
+    To use:
+    1. Initialize an instance of this class in a plugin.
+    2. Add tornadoplot.layout to the plugin layout
+    3. Register a callback that writes a json dump to tornadoplot.storage_id
+    The format of the json dump must be ('ENSEMBLE' and 'data' are mandatory, the others optional):
+    {'ENSEMBLE': name of ensemble,
+     'data': 2d array of realizations / response values
+     'number_format' (str): Format of the numeric part based on the Python Format Specification
+      Mini-Language e.g. '#.3g' for 3 significant digits, '.2f' for two decimals, or '.0f' for no
+      decimals.
+     'unit' (str): String to append at the end as a unit.
+     'spaced' (bool): Include a space between last numerical digit and SI-prefix.
+     'locked_si_prefix' (str or int): Lock the SI prefix to either a string (e.g. 'm' (milli) or 'M'
+      (mega)), or an integer which is the base 10 exponent (e.g. 3 for kilo, -3 for milli).
+    }
 
-Mouse events:
-The current case at mouse cursor can be retrieved by registering a callback
-that reads from  `tornadoplot.click_id` if `allow_click` has been specified at initialization.
+    Mouse events:
+    The current case at mouse cursor can be retrieved by registering a callback
+    that reads from  `tornadoplot.click_id` if `allow_click` has been specified at initialization.
 
 
-* `realizations`: Dataframe of realizations with corresponding sensitivity cases
-* `reference`: Which sensitivity to use as reference.
-* `allow_click`: Registers a callback to store current data on mouse click
-
-"""
+    * `realizations`: Dataframe of realizations with corresponding sensitivity cases
+    * `reference`: Which sensitivity to use as reference.
+    * `allow_click`: Registers a callback to store current data on mouse click
+    """
 
     def __init__(
-        self, app, realizations, reference="rms_seed", allow_click=False,
+        self,
+        app: dash.Dash,
+        realizations: pd.DataFrame,
+        reference: str = "rms_seed",
+        allow_click: bool = False,
     ):
 
         self.realizations = realizations
@@ -71,14 +74,17 @@ that reads from  `tornadoplot.click_id` if `allow_click` has been specified at i
         self.plotly_theme = app.webviz_settings["theme"].plotly_theme
         self.set_callbacks(app)
 
-    def ids(self, element):
+    def ids(self, element: str) -> str:
         """Generate unique id for dom element"""
         return f"{element}-id-{self.uid}"
 
     @property
-    def tour_steps(self):
+    def tour_steps(self) -> List[Dict[str, Any]]:
         return [
-            {"id": self.ids("tornado-graph"), "content": ("Shows tornado plot."),},
+            {
+                "id": self.ids("tornado-graph"),
+                "content": ("Shows tornado plot."),
+            },
             {
                 "id": self.ids("reference"),
                 "content": (
@@ -104,22 +110,22 @@ that reads from  `tornadoplot.click_id` if `allow_click` has been specified at i
         ]
 
     @property
-    def storage_id(self):
+    def storage_id(self) -> str:
         """The id of the dcc.Store component that holds the tornado data"""
         return self.ids("storage")
 
     @property
-    def click_id(self):
+    def click_id(self) -> str:
         """The id of the dcc.Store component that holds click data"""
         return self.ids("click-store")
 
     @property
-    def high_low_storage_id(self):
+    def high_low_storage_id(self) -> str:
         """The id of the dcc.Store component that holds click data"""
         return self.ids("high-low-storage")
 
     @staticmethod
-    def set_grid_layout(columns):
+    def set_grid_layout(columns: Union[str, List[str]]) -> Dict[str, str]:
         return {
             "display": "grid",
             "alignContent": "space-around",
@@ -128,7 +134,7 @@ that reads from  `tornadoplot.click_id` if `allow_click` has been specified at i
         }
 
     @property
-    def layout(self):
+    def layout(self) -> html.Div:
         return html.Div(
             [
                 html.Div(
@@ -140,7 +146,10 @@ that reads from  `tornadoplot.click_id` if `allow_click` has been specified at i
                         ),
                         html.Div(
                             style=self.set_grid_layout("1fr 1fr"),
-                            children=[html.Label("Reference:"), html.Label("Scale:"),],
+                            children=[
+                                html.Label("Reference:"),
+                                html.Label("Scale:"),
+                            ],
                         ),
                         html.Div(
                             style=self.set_grid_layout("1fr 1fr"),
@@ -152,6 +161,8 @@ that reads from  `tornadoplot.click_id` if `allow_click` has been specified at i
                                     ],
                                     value=self.initial_reference,
                                     clearable=False,
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                                 dcc.Dropdown(
                                     id=self.ids("scale"),
@@ -161,6 +172,8 @@ that reads from  `tornadoplot.click_id` if `allow_click` has been specified at i
                                     ],
                                     value="Percentage",
                                     clearable=False,
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                             ],
                         ),
@@ -173,6 +186,8 @@ that reads from  `tornadoplot.click_id` if `allow_click` has been specified at i
                                 },
                             ],
                             value=[],
+                            persistence=True,
+                            persistence_type="session",
                         ),
                         html.Details(
                             open=False,
@@ -186,6 +201,8 @@ that reads from  `tornadoplot.click_id` if `allow_click` has been specified at i
                                     value=self.sensnames,
                                     multi=True,
                                     size=min(10, len(self.sensnames)),
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                             ],
                         ),
@@ -202,15 +219,17 @@ that reads from  `tornadoplot.click_id` if `allow_click` has been specified at i
                             id=self.ids("tornado-graph"),
                             config={"displayModeBar": False},
                         ),
-                        dcc.Store(id=self.ids("storage")),
-                        dcc.Store(id=self.ids("click-store")),
-                        dcc.Store(id=self.ids("high-low-storage")),
+                        dcc.Store(id=self.ids("storage"), storage_type="session"),
+                        dcc.Store(id=self.ids("click-store"), storage_type="session"),
+                        dcc.Store(
+                            id=self.ids("high-low-storage"), storage_type="session"
+                        ),
                     ],
                 )
             ]
         )
 
-    def set_callbacks(self, app):
+    def set_callbacks(self, app: dash.Dash) -> None:
         @app.callback(
             [
                 Output(self.ids("tornado-graph"), "figure"),
@@ -224,7 +243,13 @@ that reads from  `tornadoplot.click_id` if `allow_click` has been specified at i
                 Input(self.ids("sens_filter"), "value"),
             ],
         )
-        def _calc_tornado(reference, scale, cutbyref, data, sens_filter):
+        def _calc_tornado(
+            reference: str,
+            scale: str,
+            cutbyref: str,
+            data: Union[str, bytes, bytearray],
+            sens_filter: List[str],
+        ) -> Tuple[dict, dict]:
             if not data:
                 raise PreventUpdate
             data = json.loads(data)
@@ -260,7 +285,7 @@ that reads from  `tornadoplot.click_id` if `allow_click` has been specified at i
                     Input(self.ids("reset"), "n_clicks"),
                 ],
             )
-            def _save_click_data(data, nclicks):
+            def _save_click_data(data: dict, nclicks: Optional[int]) -> str:
                 if dash.callback_context.triggered is None:
                     raise PreventUpdate
 
@@ -269,7 +294,11 @@ that reads from  `tornadoplot.click_id` if `allow_click` has been specified at i
                 if ctx == self.ids("reset") and nclicks:
 
                     return json.dumps(
-                        {"real_low": [], "real_high": [], "sens_name": None,}
+                        {
+                            "real_low": [],
+                            "real_high": [],
+                            "sens_name": None,
+                        }
                     )
                 try:
                     real_low = data["points"][0]["customdata"]
@@ -286,18 +315,19 @@ that reads from  `tornadoplot.click_id` if `allow_click` has been specified at i
                     raise PreventUpdate from exc
 
 
-def scale_to_ref(value, ref, scale):
+def scale_to_ref(value: float, ref: float, scale: str) -> float:
     value_ref = value - ref
     if scale == "Percentage":
         value_ref = (100 * (value_ref / ref)) if ref != 0 else 0
     return value_ref
 
 
-def sort_by_max(tornadotable):
+def sort_by_max(tornadotable: pd.DataFrame) -> pd.DataFrame:
+    # TODO(Ruben): This mypy error seems to be a false positive?
     """ Sorts table based on max(abs('low', 'high')) """
     tornadotable["max"] = (
         tornadotable[["low", "high"]]
-        .apply(lambda x: max(x.min(), x.max(), key=abs), axis=1)
+        .apply(lambda x: max(x.min(), x.max(), key=abs), axis=1)  # type: ignore
         .abs()
     )
     df_sorted = tornadotable.sort_values("max", ascending=True)
@@ -305,7 +335,7 @@ def sort_by_max(tornadotable):
     return df_sorted
 
 
-def cut_by_ref(tornadotable, refname):
+def cut_by_ref(tornadotable: pd.DataFrame, refname: str) -> pd.DataFrame:
     """ Removes sensitivities smaller than reference sensitivity from table """
     maskref = tornadotable.sensname == refname
     reflow = tornadotable[maskref].low.abs()
@@ -323,19 +353,20 @@ def cut_by_ref(tornadotable, refname):
 
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
 # pylint: disable=too-many-arguments
+# pylint: disable=too-many-locals
 def tornado_plot(
-    realizations,
-    data,
-    sens_filter,
-    plotly_theme,
-    reference="rms_seed",
-    scale="Percentage",
-    cutbyref=True,
-    number_format="",
-    unit="",
-    spaced=True,
-    locked_si_prefix=None,
-):  # pylint: disable=too-many-locals
+    realizations: pd.DataFrame,
+    data: pd.DataFrame,
+    sens_filter: List[str],
+    plotly_theme: Dict[str, Any],
+    reference: str = "rms_seed",
+    scale: str = "Percentage",
+    cutbyref: bool = True,
+    number_format: str = "",
+    unit: str = "",
+    spaced: bool = True,
+    locked_si_prefix: Optional[int] = None,
+) -> Tuple[dict, dict]:
 
     # Raise key error if no senscases, i.e. the ensemble has no design matrix
     if list(realizations["SENSCASE"].unique()) == [None]:
@@ -350,6 +381,8 @@ def tornado_plot(
 
     df = calc_tornado_df(data, realizations, ref_avg, scale)
 
+    locked_si_prefix_relative: Optional[int]
+
     # Drops sensitivities smaller than reference if specified
     if cutbyref and df["sensname"].str.contains(reference).any():
         df = cut_by_ref(df, reference)
@@ -390,7 +423,10 @@ def tornado_plot(
                 if reals
                 else None
                 for x, label, val, reals in zip(
-                    df["low_tooltip"], df["low_label"], df["true_low"], df["low_reals"],
+                    df["low_tooltip"],
+                    df["low_label"],
+                    df["true_low"],
+                    df["low_reals"],
                 )
             ],
             hoverinfo="text",
@@ -422,7 +458,7 @@ def tornado_plot(
             orientation="h",
         ),
     ]
-    layout = {}
+    layout: Dict[str, Any] = {}
     layout.update(plotly_theme["layout"])
     layout.update(
         {
@@ -474,7 +510,9 @@ def tornado_plot(
 
 
 # pylint: disable=too-many-locals
-def calc_tornado_df(data, realizations, ref_avg, scale):
+def calc_tornado_df(
+    data: pd.DataFrame, realizations: pd.DataFrame, ref_avg: float, scale: str
+) -> pd.DataFrame:
     # Group by sensitivity name/case and calculate average values for each case
     arr = []
 
@@ -587,7 +625,7 @@ def calc_tornado_df(data, realizations, ref_avg, scale):
     return pd.DataFrame(arr2)
 
 
-def calc_low_base(low, high):
+def calc_low_base(low: float, high: float) -> float:
     """
     From the low and high value of a parameter,
     calculates the base (starting x value) of the
@@ -604,7 +642,7 @@ def calc_low_base(low, high):
     return low
 
 
-def calc_high_base(low, high):
+def calc_high_base(low: float, high: float) -> float:
     """
     From the low and high value of a parameter,
     calculates the base (starting x value) of the bar
@@ -621,7 +659,7 @@ def calc_high_base(low, high):
     return high
 
 
-def calc_high_x(low, high):
+def calc_high_x(low: float, high: float) -> float:
     """
     From the low and high value of a parameter,
     calculates the x-value (length of bar) of the bar
@@ -636,10 +674,10 @@ def calc_high_x(low, high):
     if high > 0:
         base = max(0, low)
         return high - base
-    return 0
+    return 0.0
 
 
-def calc_low_x(low, high):
+def calc_low_x(low: float, high: float) -> float:
     """
     From the low and high value of a parameter,
     calculates the x-value (length of bar) of the bar
@@ -654,10 +692,10 @@ def calc_low_x(low, high):
     if low < 0:
         base = min(0, high)
         return low - base
-    return 0
+    return 0.0
 
 
-def printable_int_list(integer_list: Optional[List[int]]):
+def printable_int_list(integer_list: Optional[List[int]]) -> str:
     """Creates a string out of a list of integers.
     The string gives a range x-y if all the integers between and
     including x and y are in the list, otherwise separated by commas.
