@@ -1,12 +1,13 @@
-import os
+# pylint: disable=too-many-lines
+from typing import List, Tuple, Callable
 import io
 import json
 from uuid import uuid4
 from pathlib import Path
+import defusedxml.ElementTree as ET
 
 import numpy as np
 import xtgeo
-from bs4 import BeautifulSoup
 import dash
 import dash_table
 from dash.dependencies import Input, Output, State
@@ -30,20 +31,22 @@ from ._huv_table import FilterTable
 class HorizonUncertaintyViewer(WebvizPluginABC):
     """Visualizes depth uncertainty for surfaces in map view and cross section view.
 
-The cross section is defined by wellfiles and surfacefiles or a polyline.
-Polylines are drawn interactivly in map view.
+    The cross section is defined by wellfiles and surfacefiles or a polyline.
+    Polylines are drawn interactivly in map view.
 
-!> The plugin reads information from a COHIBA model file.
+    !> The plugin reads information from a COHIBA model file.
 
-* **`basedir`:** Path to folder with model_file.xml.
-   Make sure that the folder has the same format as a COHIBA folder.
-* **`planned_wells_dir`:** Path to folder with planned well files.
-   Make sure that all planned wells have format 'ROXAR RMS well'.
-"""
+    * **`basedir`:** Path to folder with model_file.xml.
+       Make sure that the folder has the same format as a COHIBA folder.
+    * **`planned_wells_dir`:** Path to folder with planned well files.
+       Make sure that all planned wells have format 'ROXAR RMS well'."""
 
     # pylint: disable=too-many-locals,too-many-instance-attributes
     def __init__(
-        self, app, basedir: Path = None, planned_wells_dir: Path = None,
+        self,
+        app: dash.Dash,
+        basedir: Path,
+        planned_wells_dir: Path = None,
     ):
 
         super().__init__()
@@ -56,10 +59,9 @@ Polylines are drawn interactivly in map view.
 
         self.basedir = basedir
         self.planned_wells_dir = planned_wells_dir
-
-        with open(get_path(basedir / "model_file.xml"), "r") as file:
-            self.modelfile = BeautifulSoup(file, "xml")
-        self.surfaces = load_surfaces(basedir, self.modelfile)
+        self.modelfile_path = basedir / "model_file.xml"
+        self.modelfile = get_path(self.modelfile_path)
+        self.surfaces = load_surfaces(basedir, self.modelfile_path)
         self.planned_wellfiles = (
             json.load(find_files(planned_wells_dir, "*.txt"))
             if planned_wells_dir
@@ -68,6 +70,7 @@ Polylines are drawn interactivly in map view.
         self.wellfiles = json.load(find_files(basedir / "input" / "welldata", "*.txt"))
         self.wellfiles = [str(get_path(Path(w))) for w in self.wellfiles]
         self.allfiles = json.load(find_files(basedir))
+        self.allfiles.append(self.modelfile_path)
         self.allfiles += self.planned_wellfiles
         self.planned_wellfiles = [
             str(get_path(Path(w))) for w in self.planned_wellfiles
@@ -111,15 +114,21 @@ Polylines are drawn interactivly in map view.
         self.state = {"switch": False}
         self.layers_state = []
 
-    def ids(self, element):
+    def ids(self, element: str) -> str:
         return f"{element}-id-{self.uid}"
 
     @property
-    def cross_section_graph_layout(self):
-        return html.Div(children=[wcc.Graph(id=self.ids("xsec-view"),)])
+    def cross_section_graph_layout(self) -> html.Div:
+        return html.Div(
+            children=[
+                wcc.Graph(
+                    id=self.ids("xsec-view"),
+                )
+            ]
+        )
 
     @property
-    def cross_section_widgets_layout(self):
+    def cross_section_widgets_layout(self) -> html.Div:
         return html.Div(
             children=[
                 html.Div(
@@ -146,6 +155,8 @@ Polylines are drawn interactivly in map view.
                                             id=self.ids("all-surfaces-checkbox"),
                                             options=[{"label": "all", "value": "True"}],
                                             value=["True"],
+                                            persistence=True,
+                                            persistence_type="session",
                                         ),
                                         dcc.Checklist(
                                             id=self.ids("surfaces-checklist"),
@@ -154,6 +165,8 @@ Polylines are drawn interactivly in map view.
                                                 for name in self.surfacenames
                                             ],
                                             value=self.surfacenames,
+                                            persistence=True,
+                                            persistence_type="session",
                                         ),
                                         dcc.Checklist(
                                             id=self.ids("surfaces-de-checklist"),
@@ -166,6 +179,8 @@ Polylines are drawn interactivly in map view.
                                                 for name in self.surfacenames
                                             ],
                                             value=self.surfacenames,
+                                            persistence=True,
+                                            persistence_type="session",
                                         ),
                                     ],
                                 ),
@@ -209,6 +224,8 @@ Polylines are drawn interactivly in map view.
                                             id=self.ids("all-well-settings-checkbox"),
                                             options=[{"label": "all", "value": "True"}],
                                             value=["True"],
+                                            persistence=True,
+                                            persistence_type="session",
                                         ),
                                         dcc.Checklist(
                                             id=self.ids("well-settings-checklist"),
@@ -231,6 +248,8 @@ Polylines are drawn interactivly in map view.
                                                 "zonation_points",
                                                 "conditional_points",
                                             ],
+                                            persistence=True,
+                                            persistence_type="session",
                                         ),
                                     ],
                                 ),
@@ -289,6 +308,8 @@ Polylines are drawn interactivly in map view.
                                     value=str(self.wellfiles[0]),
                                     clearable=False,
                                     disabled=False,
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                             ]
                         ),
@@ -311,7 +332,7 @@ Polylines are drawn interactivly in map view.
         )
 
     @property
-    def target_points_tab_layout(self):
+    def target_points_tab_layout(self) -> dash_table.DataTable:
         df = self.df_well_target_points.get_targetpoints_df()
         return dash_table.DataTable(
             id=self.ids("target-point-table"),
@@ -322,7 +343,7 @@ Polylines are drawn interactivly in map view.
         )
 
     @property
-    def well_points_tab_layout(self):
+    def well_points_tab_layout(self) -> html.Div:
         return html.Div(
             [
                 dbc.Button("Table Settings", id=self.ids("button-open-table-settings")),
@@ -332,7 +353,10 @@ Polylines are drawn interactivly in map view.
                         dbc.ModalBody(
                             children=[
                                 html.Label(
-                                    style={"font-weight": "bold", "textAlign": "Left",},
+                                    style={
+                                        "font-weight": "bold",
+                                        "textAlign": "Left",
+                                    },
                                     children="Select Table Columns",
                                 ),
                                 dcc.Checklist(
@@ -357,6 +381,8 @@ Polylines are drawn interactivly in map view.
                                         "Deleted",
                                         "Residual",
                                     ],
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                             ],
                         ),
@@ -386,7 +412,7 @@ Polylines are drawn interactivly in map view.
         )
 
     @property
-    def left_flexbox_layout(self):
+    def left_flexbox_layout(self) -> html.Div:
         return html.Div(
             children=[
                 wcc.FlexBox(
@@ -399,6 +425,8 @@ Polylines are drawn interactivly in map view.
                             ],
                             id=self.ids("map-table-radioitems"),
                             value="map-view",
+                            persistence=True,
+                            persistence_type="session",
                         )
                     ],
                     style={"padding": "5px"},
@@ -414,7 +442,7 @@ Polylines are drawn interactivly in map view.
         )
 
     @property
-    def map_view_layout(self):
+    def map_view_layout(self) -> html.Div:
         return html.Div(
             children=[
                 wcc.FlexBox(
@@ -436,13 +464,19 @@ Polylines are drawn interactivly in map view.
                                     ],
                                     value=self.surfacenames[0],
                                     clearable=False,
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                             ]
                         ),
                     ],
                 ),
                 html.Div(
-                    style={"marginTop": "0px", "height": "800px", "zIndex": -9999,},
+                    style={
+                        "marginTop": "0px",
+                        "height": "800px",
+                        "zIndex": -9999,
+                    },
                     children=[
                         # pylint: disable=no-member
                         webviz_subsurface_components.LeafletMap(
@@ -471,13 +505,16 @@ Polylines are drawn interactivly in map view.
         )
 
     @property
-    def table_view_layout(self):
+    def table_view_layout(self) -> html.Div:
         df = self.xsec.get_intersection_dataframe(self.wells[self.wellfiles[0]])
         return html.Div(
             children=[
                 html.Label(
                     id=self.ids("surface-picks-label"),
-                    style={"font-weight": "bold", "textAlign": "center",},
+                    style={
+                        "font-weight": "bold",
+                        "textAlign": "center",
+                    },
                 ),
                 dash_table.DataTable(
                     id=self.ids("uncertainty-table"),
@@ -491,7 +528,7 @@ Polylines are drawn interactivly in map view.
         )
 
     @property
-    def layout(self):
+    def layout(self) -> dcc.Tabs:
         return dcc.Tabs(
             children=[
                 dcc.Tab(
@@ -520,7 +557,7 @@ Polylines are drawn interactivly in map view.
         )
 
     # pylint: disable=too-many-statements
-    def set_callbacks(self, app):
+    def set_callbacks(self, app: dash.Dash) -> None:
         @app.callback(
             Output(self.ids("layered-map"), "layers"),
             [
@@ -530,8 +567,8 @@ Polylines are drawn interactivly in map view.
             ],
         )
         def _render_map(surface_name, switch, wellfile):
-            """ Renders map view for one surface with de, dt, dte, dr, dre and depth
-                Wells marked with circles, trajectory and hillshading toggle
+            """Renders map view for one surface with de, dt, dte, dr, dre and depth
+            Wells marked with circles, trajectory and hillshading toggle
             """
             # Store layers when switching to hillshading
             if self.state["switch"] is not switch["value"]:
@@ -553,7 +590,11 @@ Polylines are drawn interactivly in map view.
                 self.surface_attributes[surface_name]["surface_dte"],
             ]
             well_layers = get_well_layers(
-                self.wells, self.planned_wells, surface_name, surfaces[0], wellfile,
+                self.wells,
+                self.planned_wells,
+                surface_name,
+                surfaces[0],
+                wellfile,
             )
             layers = get_surface_layers(switch, surface_name, surfaces)
             layers.extend(well_layers)
@@ -655,7 +696,7 @@ Polylines are drawn interactivly in map view.
             [State(self.ids("surfaces-de-checklist"), "options")],
         )
         def _disable_error_checkboxes(surface_values, de_options):
-            """ Removes ability to toggle depth error when
+            """Removes ability to toggle depth error when
             corresponding surface is disabled in graph settings modal
             """
             for i, opt in enumerate(de_options):
@@ -693,8 +734,12 @@ Polylines are drawn interactivly in map view.
 
         @app.callback(
             Output(self.ids("well-points-table-container"), "children"),
-            [Input(self.ids("button-apply-columnlist"), "n_clicks"),],
-            [State(self.ids("columns-checklist"), "value"),],  # columns list
+            [
+                Input(self.ids("button-apply-columnlist"), "n_clicks"),
+            ],
+            [
+                State(self.ids("columns-checklist"), "value"),
+            ],  # columns list
         )
         # pylint: disable=unused-variable, unused-argument
         def display_output(n_clicks, column_list):
@@ -764,7 +809,7 @@ Polylines are drawn interactivly in map view.
                 wellname = self.planned_wells[wellfile].wellname
             return f"Surface picks for {wellname}"
 
-    def add_webvizstore(self):
+    def add_webvizstore(self) -> List[Tuple[Callable, list]]:
         functions = [(get_path, [{"path": Path(fn)}]) for fn in self.allfiles]
         functions.append((find_files, [{"folder": self.basedir}]))
         if self.planned_wells_dir is not None:
@@ -781,7 +826,15 @@ Polylines are drawn interactivly in map view.
                 )
             )
         functions.append(
-            (get_surfaces, [{"basedir": self.basedir, "modelfile": self.modelfile,}],)
+            (
+                get_surfaces,
+                [
+                    {
+                        "basedir": self.basedir,
+                        "modelfile": self.modelfile_path,
+                    }
+                ],
+            )
         )
         return functions
 
@@ -792,7 +845,7 @@ def get_path(path) -> Path:
 
 
 def get_color(i):
-    """ Create a list of colors for surface layers
+    """Create a list of colors for surface layers
     Args:
         i: Index of surface layer in surfacefiles list
     Returns:
@@ -836,6 +889,7 @@ def load_surfaces(basedir: Path, modelfile):
 
 @webvizstore
 def get_surfaces(basedir: Path, modelfile) -> io.BytesIO:
+    modelfile = ET.parse(get_path(modelfile)).getroot()
     surface_types = {
         "depth": "d_",
         "depth-trend": "dt_",
@@ -843,12 +897,12 @@ def get_surfaces(basedir: Path, modelfile) -> io.BytesIO:
         "depth-trend-error": "dte_",
         "depth-residual": "dr_",
     }
-    surface_wrappers = modelfile.find_all("surface")
+    surface_wrappers = modelfile.findall(".//surface")
     surfaces = []
     for element in surface_wrappers:
         surface = {
-            "name": element.find("name").get_text(),
-            "topofzone": element.find("top-of-zone").get_text(),
+            "name": element.findtext("name"),
+            "topofzone": element.findtext("top-of-zone"),
         }
         for s_mtype, s_type in surface_types.items():
             surface[s_type] = (
@@ -860,14 +914,14 @@ def get_surfaces(basedir: Path, modelfile) -> io.BytesIO:
                         / f"{s_type}{surface['name']}.rxb"
                     )
                 )
-                if element.find(s_mtype).get_text() == "yes"
+                if element.find("output").findtext(s_mtype) == "yes"
                 else None
             )
         surfaces.append(surface)
     return io.BytesIO(json.dumps(surfaces).encode())
 
 
-def surface_to_json(surfacepath: Path):
+def surface_to_json(surfacepath: Path) -> str:
     surface = xtgeo.surface_from_file(str(surfacepath), fformat="irap_binary")
     return json.dumps(
         {
@@ -890,63 +944,68 @@ def surface_from_json(surfaceobj):
     return surface
 
 
-def extract_topofzone_names(modelfile):
-    surface_wrappers = modelfile.find_all("surface")
-    topofzone_names = []
-    for element in surface_wrappers:
-        name = element.find("top-of-zone")
-        topofzone_names.append(name.get_text())
-    return topofzone_names
+# TODO(Sigurd) Delete unused function?
+# def extract_topofzone_names(modelfile):
+#    modelfile = ET.parse(modelfile).getroot()
+#    surface_wrappers = modelfile.findall(".//surface")
+#    topofzone_names = []
+#    for element in surface_wrappers:
+#        name = element.findtext("top-of-zone")
+#        topofzone_names.append(name)
+#    return topofzone_names
 
 
-def get_well_files(basedir: Path) -> io.BytesIO:
-    well_dir = os.path.join(basedir, "input", "welldata")
-    well_dir = basedir / "input" / "welldata"
-    well_files = []
-    try:
-        for file in os.listdir(well_dir):
-            if Path(file).suffix == ".txt":
-                well_files.append(os.path.join(well_dir, file))
-    except FileNotFoundError:
-        pass
-    well_files.sort()
-    return io.BytesIO(json.dumps(well_files).encode())
+# TODO(Sigurd) Delete unused function?
+# def get_well_files(basedir: Path) -> io.BytesIO:
+#    well_dir = os.path.join(basedir, "input", "welldata")
+#    well_dir = basedir / "input" / "welldata"
+#    well_files = []
+#    try:
+#        for file in os.listdir(well_dir):
+#            if Path(file).suffix == ".txt":
+#                well_files.append(os.path.join(well_dir, file))
+#    except FileNotFoundError:
+#        pass
+#    well_files.sort()
+#    return io.BytesIO(json.dumps(well_files).encode())
 
 
-def get_target_points(basedir: Path):
+def get_target_points(basedir: Path) -> Path:
     return get_path(basedir / "output" / "log_files" / "targetpoints.csv")
 
 
-def get_well_points(basedir: Path):
+def get_well_points(basedir: Path) -> Path:
     return get_path(basedir / "output" / "log_files" / "wellpoints.csv")
 
 
 def get_zonelog_name(modelfile):
-    zonelog_wrapper = modelfile.find("zone-log-name")
-    return zonelog_wrapper.get_text()
+    modelfile = ET.parse(modelfile).getroot()
+    zonelog_wrapper = modelfile.findtext(".//zone-log-name")
+    return zonelog_wrapper
 
 
-def get_zonation_status(basedir: Path):
+def get_zonation_status(basedir: Path) -> Path:
     return get_path(basedir / "output" / "log_files" / "zonation_status.csv")
 
 
-@webvizstore
-def get_surface_files(basedir: Path, surface_names, surface_type) -> io.BytesIO:
-    surface_dir = basedir / "output" / "surfaces"
-    surface_files = [
-        surface_dir / f"{surface_type}{surface_name}.rxb"
-        for surface_name in surface_names
-    ]
-    for path in surface_files:
-        if not path.is_file():
-            surface_files = None
-    if surface_files is not None:
-        surface_files = [str(surf) for surf in surface_files]
-    return io.BytesIO(json.dumps(surface_files).encode())
+# TODO(Sigurd) Delete unused?
+# @webvizstore
+# def get_surface_files(basedir: Path, surface_names, surface_type) -> io.BytesIO:
+#    surface_dir = basedir / "output" / "surfaces"
+#    surface_files = [
+#        surface_dir / f"{surface_type}{surface_name}.rxb"
+#        for surface_name in surface_names
+#    ]
+#    for path in surface_files:
+#        if not path.is_file():
+#            surface_files = None
+#    if surface_files is not None:
+#        surface_files = [str(surf) for surf in surface_files]
+#    return io.BytesIO(json.dumps(surface_files).encode())
 
 
 @webvizstore
-def find_files(folder, pattern="*") -> io.BytesIO:
+def find_files(folder: Path, pattern: str = "*") -> io.BytesIO:
     return io.BytesIO(
         json.dumps(
             sorted(
