@@ -1,7 +1,10 @@
+from typing import List, Union, Tuple, Callable
 from uuid import uuid4
+from pathlib import Path
 
 import numpy as np
 import pandas as pd
+import dash
 import dash_html_components as html
 import dash_core_components as dcc
 import webviz_core_components as wcc
@@ -15,30 +18,31 @@ from .._datainput.fmu_input import scratch_ensemble
 
 class Widgets:
     @staticmethod
-    def dropdown_from_dict(dom_id, dictionary):
+    def dropdown_from_dict(dom_id: str, dictionary: dict) -> dcc.Dropdown:
         return dcc.Dropdown(
             id=dom_id,
             options=[{"label": k, "value": v} for k, v in dictionary.items()],
             value=list(dictionary.values())[0],
             clearable=False,
+            persistence=True,
+            persistence_type="session",
         )
 
 
 class ParameterCorrelation(WebvizPluginABC):
     """Shows parameter correlations using a correlation matrix,
-and scatter plot for any given pair of parameters.
+    and scatter plot for any given pair of parameters.
 
----
+    ---
 
-* **`ensembles`:** Which ensembles in `shared_settings` to visualize.
-* **`drop_constants`:** Drop constant parameters.
+    * **`ensembles`:** Which ensembles in `shared_settings` to visualize.
+    * **`drop_constants`:** Drop constant parameters.
 
----
-Parameter values are extracted automatically from the `parameters.txt` files in the individual
-realizations of your defined `ensembles`, using the `fmu-ensemble` library.
-"""
+    ---
+    Parameter values are extracted automatically from the `parameters.txt` files in the individual
+    realizations of your defined `ensembles`, using the `fmu-ensemble` library."""
 
-    def __init__(self, app, ensembles: list, drop_constants: bool = True):
+    def __init__(self, app: dash.Dash, ensembles: list, drop_constants: bool = True):
 
         super().__init__()
 
@@ -52,19 +56,19 @@ realizations of your defined `ensembles`, using the `fmu-ensemble` library.
         self.uid = uuid4()
         self.set_callbacks(app)
 
-    def ids(self, element):
+    def ids(self, element: str) -> str:
         """Generate unique id for dom element"""
         return f"{element}-id-{self.uid}"
 
     @property
-    def p_cols(self):
+    def p_cols(self) -> list:
         dfs = [
             get_corr_data(ens, self.drop_constants) for ens in self.ensembles.values()
         ]
-        return sorted(list(pd.concat(dfs).columns))
+        return sorted(list(pd.concat(dfs, sort=True).columns))
 
     @property
-    def matrix_plot(self):
+    def matrix_plot(self) -> html.Div:
         return html.Div(
             style={"height": "400px"},
             children=wcc.Graph(
@@ -74,19 +78,22 @@ realizations of your defined `ensembles`, using the `fmu-ensemble` library.
         )
 
     @property
-    def control_div(self):
+    def control_div(self) -> list:
         return [
             html.Div(
                 style={"padding": "5px"},
                 children=[
                     html.Label(
-                        "Parameter horizontal axis:", style={"font-weight": "bold"},
+                        "Parameter horizontal axis:",
+                        style={"font-weight": "bold"},
                     ),
                     dcc.Dropdown(
                         id=self.ids("parameter1"),
                         options=[{"label": p, "value": p} for p in self.p_cols],
                         value=self.p_cols[0],
                         clearable=False,
+                        persistence=True,
+                        persistence_type="session",
                     ),
                     Widgets.dropdown_from_dict(self.ids("ensemble-1"), self.ensembles),
                 ],
@@ -102,6 +109,8 @@ realizations of your defined `ensembles`, using the `fmu-ensemble` library.
                         options=[{"label": p, "value": p} for p in self.p_cols],
                         value=self.p_cols[0],
                         clearable=False,
+                        persistence=True,
+                        persistence_type="session",
                     ),
                     Widgets.dropdown_from_dict(self.ids("ensemble-2"), self.ensembles),
                 ],
@@ -113,18 +122,27 @@ realizations of your defined `ensembles`, using the `fmu-ensemble` library.
                     dcc.Dropdown(
                         id=self.ids("scatter-color"),
                         options=[{"label": p, "value": p} for p in self.p_cols],
+                        persistence=True,
+                        persistence_type="session",
                     ),
                 ],
             ),
             dcc.Checklist(
                 id=self.ids("density"),
                 style={"padding": "5px"},
-                options=[{"label": "Show scatterplot density", "value": "density",}],
+                options=[
+                    {
+                        "label": "Show scatterplot density",
+                        "value": "density",
+                    }
+                ],
+                persistence=True,
+                persistence_type="session",
             ),
         ]
 
     @property
-    def layout(self):
+    def layout(self) -> wcc.FlexBox:
         return wcc.FlexBox(
             children=[
                 html.Div(
@@ -158,7 +176,7 @@ realizations of your defined `ensembles`, using the `fmu-ensemble` library.
             ]
         )
 
-    def set_callbacks(self, app):
+    def set_callbacks(self, app: dash.Dash) -> None:
         @app.callback(
             Output(self.ids("matrix"), "figure"),
             [
@@ -167,7 +185,7 @@ realizations of your defined `ensembles`, using the `fmu-ensemble` library.
                 Input(self.ids("parameter2"), "value"),
             ],
         )
-        def _update_matrix(ens, param1, param2):
+        def _update_matrix(ens: str, param1: str, param2: str) -> dict:
             """Renders correlation matrix.
             Currently also re-renders matrix to update currently
             selected cell. This is not optimal, but hard to prevent
@@ -208,7 +226,14 @@ realizations of your defined `ensembles`, using the `fmu-ensemble` library.
                 Input(self.ids("density"), "value"),
             ],
         )
-        def _update_scatter(ens1, param1, ens2, param2, color, density):
+        def _update_scatter(
+            ens1: str,
+            param1: str,
+            ens2: str,
+            param2: str,
+            color: Union[str, None],
+            density: List[str],
+        ) -> dict:
             return render_scatter(
                 ens1, param1, ens2, param2, color, density, self.plotly_theme
             )
@@ -225,7 +250,7 @@ realizations of your defined `ensembles`, using the `fmu-ensemble` library.
                 Input(self.ids("ensemble-all"), "value"),
             ],
         )
-        def _update_from_click(cell_data, ens):
+        def _update_from_click(cell_data: dict, ens: str) -> List[Union[str, None]]:
             try:
                 points = cell_data["points"][0]
             # TypeError is returned if no cells are clicked
@@ -234,15 +259,15 @@ realizations of your defined `ensembles`, using the `fmu-ensemble` library.
 
             return [points["x"], points["y"], ens, ens]
 
-    def add_webvizstore(self):
+    def add_webvizstore(self) -> List[Tuple[Callable, list]]:
         return [
-            ([get_parameters, [{"ensemble_path": v} for v in self.ensembles.values()]])
+            (get_parameters, [{"ensemble_path": v} for v in self.ensembles.values()])
         ]
 
 
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
 @webvizstore
-def get_parameters(ensemble_path) -> pd.DataFrame:
+def get_parameters(ensemble_path: Path) -> pd.DataFrame:
 
     return (
         scratch_ensemble("", ensemble_path)
@@ -252,7 +277,15 @@ def get_parameters(ensemble_path) -> pd.DataFrame:
 
 
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
-def render_scatter(ens1, x_col, ens2, y_col, color, density, theme):
+def render_scatter(
+    ens1: str,
+    x_col: str,
+    ens2: str,
+    y_col: str,
+    color: Union[str, None],
+    density: bool,
+    theme: dict,
+) -> dict:
     if ens1 == ens2:
         real_text = [f"Realization:{r}" for r in get_parameters(ens1)["REAL"]]
     else:
@@ -347,7 +380,7 @@ def render_scatter(ens1, x_col, ens2, y_col, color, density, theme):
 
 
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
-def get_corr_data(ensemble_path, drop_constants=True):
+def get_corr_data(ensemble_path: str, drop_constants: bool = True) -> pd.DataFrame:
     """
     if drop_constants:
     .dropna() removes undefined entries in correlation matrix after
@@ -358,6 +391,13 @@ def get_corr_data(ensemble_path, drop_constants=True):
     """
     data = get_parameters(ensemble_path)
 
+    # Necessary to drop constant before correlations due to
+    # https://github.com/pandas-dev/pandas/issues/37448
+    if drop_constants is True:
+        for col in data.columns:
+            if len(data[col].unique()) == 1:
+                data = data.drop(col, axis=1)
+
     return (
         data.corr()
         if not drop_constants
@@ -368,7 +408,7 @@ def get_corr_data(ensemble_path, drop_constants=True):
 
 
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
-def render_matrix(ensemble_path, theme, drop_constants=True):
+def render_matrix(ensemble_path: str, theme: dict, drop_constants: bool = True) -> dict:
     corrdf = get_corr_data(ensemble_path, drop_constants)
     # pylint: disable=no-member
     corrdf = corrdf.mask(np.tril(np.ones(corrdf.shape)).astype(np.bool))
@@ -388,15 +428,23 @@ def render_matrix(ensemble_path, theme, drop_constants=True):
             "paper_bgcolor": "rgba(0,0,0,0)",
             "plot_bgcolor": "rgba(0,0,0,0)",
             "margin": {"t": 50, "b": 50},
-            "xaxis": {"ticks": "", "showticklabels": False, "showgrid": False,},
-            "yaxis": {"ticks": "", "showticklabels": False, "showgrid": False,},
+            "xaxis": {
+                "ticks": "",
+                "showticklabels": False,
+                "showgrid": False,
+            },
+            "yaxis": {
+                "ticks": "",
+                "showticklabels": False,
+                "showgrid": False,
+            },
         },
     )
 
     return {"data": [data], "layout": layout}
 
 
-def theme_layout(theme, specific_layout):
+def theme_layout(theme: dict, specific_layout: dict) -> dict:
     layout = {}
     layout.update(theme["layout"])
     layout.update(specific_layout)
