@@ -10,7 +10,7 @@ from webviz_config import WebvizPluginABC
 from webviz_config.common_cache import CACHE
 from webviz_config.webviz_store import webvizstore
 
-from .._datainput.fmu_input import load_parameters, load_csv, load_smry
+from webviz_subsurface._models import EnsembleSetModel
 from .._utils.parameter_response import filter_and_sum_responses
 
 
@@ -183,25 +183,21 @@ folder, to avoid risk of not extracting the right data.
                     'Incorrect arguments. Either provide "response_csv" or '
                     '"ensembles and/or response_file".'
                 )
-            self.ens_paths = {
-                ens: app.webviz_settings["shared_settings"]["scratch_ensembles"][ens]
-                for ens in ensembles
-            }
-            self.parameterdf = load_parameters(
-                ensemble_paths=self.ens_paths, ensemble_set_name="EnsembleSet"
+            self.emodel = EnsembleSetModel(
+                ensemble_paths={
+                    ens: app.webviz_settings["shared_settings"]["scratch_ensembles"][
+                        ens
+                    ]
+                    for ens in ensembles
+                }
             )
+            self.parameterdf = self.emodel.load_parameters()
             if not self.no_responses:
                 if self.response_file:
-                    self.responsedf = load_csv(
-                        ensemble_paths=self.ens_paths,
-                        csv_file=response_file,
-                        ensemble_set_name="EnsembleSet",
-                    )
+                    self.responsedf = self.emodel.load_csv(csv_file=response_file)
                 else:
-                    self.responsedf = load_smry(
-                        ensemble_paths=self.ens_paths,
-                        column_keys=self.column_keys,
-                        time_index=self.time_index,
+                    self.responsedf = self.emodel.load_smry(
+                        time_index=self.time_index, column_keys=self.column_keys
                     )
                     self.response_filters["DATE"] = "single"
         else:
@@ -325,6 +321,8 @@ folder, to avoid risk of not extracting the right data.
                 clearable=False,
                 value=self.responses[0],
                 style={"marginBottom": "20px"},
+                persistence=True,
+                persistence_type="session",
             ),
         ]
 
@@ -338,6 +336,8 @@ folder, to avoid risk of not extracting the right data.
                         value=values,
                         multi=True,
                         size=min(20, len(values)),
+                        persistence=True,
+                        persistence_type="session",
                     )
                 elif col_type == "single":
                     selector = dcc.Dropdown(
@@ -346,8 +346,17 @@ folder, to avoid risk of not extracting the right data.
                         value=values[0],
                         multi=False,
                         clearable=False,
+                        persistence=True,
+                        persistence_type="session",
                     )
-                children.append(html.Div(children=[html.Label(col_name), selector,]))
+                children.append(
+                    html.Div(
+                        children=[
+                            html.Label(col_name),
+                            selector,
+                        ]
+                    )
+                )
         return [
             html.Div(
                 id=self.uuid("view_response"),
@@ -374,6 +383,8 @@ folder, to avoid risk of not extracting the right data.
                             ],
                             value="ensemble",
                             labelStyle={"display": "inline-block"},
+                            persistence=True,
+                            persistence_type="session",
                         ),
                     ]
                 )
@@ -388,13 +399,17 @@ folder, to avoid risk of not extracting the right data.
                 multi=True,
                 value=self.ensembles,
                 size=min(len(self.ensembles), 10),
+                persistence=True,
+                persistence_type="session",
             ),
             html.Label(
                 children=[
                     html.Span(
                         "Parameters:",
                         id=self.uuid("parameters"),
-                        style={"font-weight": "bold",},
+                        style={
+                            "font-weight": "bold",
+                        },
                     ),
                     dcc.RadioItems(
                         id=self.uuid("exclude_include"),
@@ -405,6 +420,8 @@ folder, to avoid risk of not extracting the right data.
                         value="exc",
                         labelStyle={"display": "inline-block"},
                         style={"fontSize": ".80em"},
+                        persistence=True,
+                        persistence_type="session",
                     ),
                 ]
             ),
@@ -419,6 +436,8 @@ folder, to avoid risk of not extracting the right data.
                     "fontSize": ".80em",
                     "overflowX": "auto",
                 },
+                persistence=True,
+                persistence_type="session",
             ),
         ]
 
@@ -433,7 +452,10 @@ folder, to avoid risk of not extracting the right data.
                     children=(self.control_layout + self.response_layout),
                 ),
                 html.Div(
-                    style={"flex": 3}, children=wcc.Graph(id=self.uuid("parcoords"),),
+                    style={"flex": 3},
+                    children=wcc.Graph(
+                        id=self.uuid("parcoords"),
+                    ),
                 ),
             ],
         )
@@ -472,7 +494,8 @@ folder, to avoid risk of not extracting the right data.
 
     def set_callbacks(self, app):
         @app.callback(
-            Output(self.uuid("parcoords"), "figure"), self.parcoord_inputs,
+            Output(self.uuid("parcoords"), "figure"),
+            self.parcoord_inputs,
         )
         def _update_parcoord(ens, exc_inc, parameter_list, *opt_args):
             """Updates parallel coordinates plot
@@ -515,6 +538,7 @@ folder, to avoid risk of not extracting the right data.
                     filteroptions=filteroptions,
                     aggregation=self.aggregation,
                 )
+
                 # Renaming to make it clear in plot.
                 responsedf.rename(
                     columns={response: f"Response: {response}"}, inplace=True
@@ -556,44 +580,35 @@ folder, to avoid risk of not extracting the right data.
     def add_webvizstore(self):
         functions = []
         if self.parameter_csv:
-            functions.append((read_csv, [{"csv_file": self.parameter_csv,}],))
+            functions.append(
+                (
+                    read_csv,
+                    [
+                        {
+                            "csv_file": self.parameter_csv,
+                        }
+                    ],
+                )
+            )
             if self.response_csv:
-                functions.append((read_csv, [{"csv_file": self.response_csv,}],))
+                functions.append(
+                    (
+                        read_csv,
+                        [
+                            {
+                                "csv_file": self.response_csv,
+                            }
+                        ],
+                    )
+                )
         else:
-            functions.append((load_parameters, [{"ensemble_paths": self.ens_paths,}],),)
-            if not self.no_responses:
-                if self.response_file:
-                    functions.append(
-                        (
-                            load_csv,
-                            [
-                                {
-                                    "ensemble_paths": self.ens_paths,
-                                    "csv_file": self.response_file,
-                                }
-                            ],
-                        ),
-                    )
-                else:
-                    functions.append(
-                        (
-                            load_smry,
-                            [
-                                {
-                                    "ensemble_paths": self.ens_paths,
-                                    "column_keys": self.column_keys,
-                                    "time_index": self.time_index,
-                                }
-                            ],
-                        ),
-                    )
+            functions.extend(self.emodel.webvizstore)
 
         return functions
 
 
 def render_parcoord(plot_df, theme, colormap, color_col, ens, mode, params, response):
-    """Renders parallel coordinates plot
-    """
+    """Renders parallel coordinates plot"""
     colormap = (
         colormap if mode == "ensemble" else theme.plotly_theme["layout"]["colorway"]
     )
