@@ -13,7 +13,8 @@ from webviz_config.common_cache import CACHE
 from webviz_config import WebvizPluginABC
 from webviz_config.utils import calculate_slider_step
 
-from .._datainput.fmu_input import load_parameters, load_csv, load_smry
+from webviz_subsurface._models import EnsembleSetModel
+from .._datainput.fmu_input import load_parameters, load_csv
 
 
 class ParameterResponseCorrelation(WebvizPluginABC):
@@ -172,8 +173,8 @@ folder, to avoid risk of not extracting the right data.
                     ensemble_set_name="EnsembleSet",
                 )
             else:
-                self.responsedf = load_smry(
-                    ensemble_paths=self.ens_paths,
+                self.emodel = EnsembleSetModel(ensemble_paths=self.ens_paths)
+                self.responsedf = self.emodel.load_smry(
                     column_keys=self.column_keys,
                     time_index=self.time_index,
                 )
@@ -235,8 +236,14 @@ folder, to avoid risk of not extracting the right data.
                     "in the correlation chart."
                 ),
             },
-            {"id": self.ids("ensemble"), "content": ("Select the active ensemble."),},
-            {"id": self.ids("responses"), "content": ("Select the active response."),},
+            {
+                "id": self.ids("ensemble"),
+                "content": ("Select the active ensemble."),
+            },
+            {
+                "id": self.ids("responses"),
+                "content": ("Select the active response."),
+            },
         ]
 
         return steps
@@ -303,6 +310,8 @@ folder, to avoid risk of not extracting the right data.
                     value=values,
                     multi=True,
                     size=min(20, len(values)),
+                    persistence=True,
+                    persistence_type="session",
                 )
             elif col_type == "single":
                 selector = dcc.Dropdown(
@@ -311,12 +320,21 @@ folder, to avoid risk of not extracting the right data.
                     value=values[0],
                     multi=False,
                     clearable=False,
+                    persistence=True,
+                    persistence_type="session",
                 )
             elif col_type == "range":
                 selector = make_range_slider(domid, self.responsedf[col_name], col_name)
             else:
                 return children
-            children.append(html.Div(children=[html.Label(col_name), selector,]))
+            children.append(
+                html.Div(
+                    children=[
+                        html.Label(col_name),
+                        selector,
+                    ]
+                )
+            )
 
         return children
 
@@ -334,6 +352,8 @@ folder, to avoid risk of not extracting the right data.
                         ],
                         clearable=False,
                         value=self.ensembles[0],
+                        persistence=True,
+                        persistence_type="session",
                     ),
                 ]
             ),
@@ -347,6 +367,8 @@ folder, to avoid risk of not extracting the right data.
                         ],
                         clearable=False,
                         value=self.responses[0],
+                        persistence=True,
+                        persistence_type="session",
                     ),
                 ]
             ),
@@ -362,7 +384,9 @@ folder, to avoid risk of not extracting the right data.
                     style={"flex": 3},
                     children=[
                         wcc.Graph(self.ids("correlation-graph")),
-                        dcc.Store(id=self.ids("initial-parameter")),
+                        dcc.Store(
+                            id=self.ids("initial-parameter"), storage_type="session"
+                        ),
                     ],
                 ),
                 html.Div(
@@ -503,10 +527,25 @@ folder, to avoid risk of not extracting the right data.
     def add_webvizstore(self):
         if self.parameter_csv and self.response_csv:
             return [
-                (read_csv, [{"csv_file": self.parameter_csv,}],),
-                (read_csv, [{"csv_file": self.response_csv,}],),
+                (
+                    read_csv,
+                    [
+                        {
+                            "csv_file": self.parameter_csv,
+                        }
+                    ],
+                ),
+                (
+                    read_csv,
+                    [
+                        {
+                            "csv_file": self.response_csv,
+                        }
+                    ],
+                ),
             ]
-        return [
+
+        functions = [
             (
                 load_parameters,
                 [
@@ -516,28 +555,23 @@ folder, to avoid risk of not extracting the right data.
                     }
                 ],
             ),
-            (
-                load_csv,
-                [
-                    {
-                        "ensemble_paths": self.ens_paths,
-                        "csv_file": self.response_file,
-                        "ensemble_set_name": "EnsembleSet",
-                    }
-                ],
-            )
-            if self.response_file
-            else (
-                load_smry,
-                [
-                    {
-                        "ensemble_paths": self.ens_paths,
-                        "column_keys": self.column_keys,
-                        "time_index": self.time_index,
-                    }
-                ],
-            ),
         ]
+        if self.response_file:
+            functions.append(
+                (
+                    load_csv,
+                    [
+                        {
+                            "ensemble_paths": self.ens_paths,
+                            "csv_file": self.response_file,
+                            "ensemble_set_name": "EnsembleSet",
+                        }
+                    ],
+                )
+            )
+        else:
+            functions.extend(self.emodel.webvizstore)
+        return functions
 
 
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
@@ -555,7 +589,11 @@ def filter_and_sum_responses(
 
 
 def _filter_and_sum_responses(
-    dframe, ensemble, response, filteroptions=None, aggregation="sum",
+    dframe,
+    ensemble,
+    response,
+    filteroptions=None,
+    aggregation="sum",
 ):
     """Filter response dataframe for the given ensemble
     and optional filter columns. Returns dataframe grouped and
@@ -658,10 +696,22 @@ def make_distribution_plot(df, parameter, response, theme):
         1,
     )
     fig.add_trace(
-        {"type": "histogram", "x": df[parameter], "showlegend": False,}, 3, 1,
+        {
+            "type": "histogram",
+            "x": df[parameter],
+            "showlegend": False,
+        },
+        3,
+        1,
     )
     fig.add_trace(
-        {"type": "histogram", "x": df[response], "showlegend": False,}, 3, 2,
+        {
+            "type": "histogram",
+            "x": df[response],
+            "showlegend": False,
+        },
+        3,
+        2,
     )
     fig["layout"].update(
         theme_layout(
@@ -669,7 +719,9 @@ def make_distribution_plot(df, parameter, response, theme):
             {
                 "height": 800,
                 "bargap": 0.05,
-                "xaxis": {"title": parameter,},
+                "xaxis": {
+                    "title": parameter,
+                },
                 "yaxis": {"title": response},
                 "xaxis2": {"title": parameter},
                 "xaxis3": {"title": response},
