@@ -1,6 +1,6 @@
 # pylint: disable=too-many-lines
-import warnings
 from typing import Optional, Union
+import warnings
 from pathlib import Path
 
 import numpy as np
@@ -14,6 +14,7 @@ from webviz_config import WebvizPluginABC
 
 from .._datainput.relative_permeability import load_satfunc, load_scal_recommendation
 from .._datainput.fmu_input import load_csv
+from .._utils.colors import hex_to_rgba
 
 
 class RelativePermeability(WebvizPluginABC):
@@ -25,9 +26,9 @@ class RelativePermeability(WebvizPluginABC):
 * **`relpermfile`:** Local path to a csvfile in each realization with dumped relperm data.
 * **`scalfile`:** Path to a reference file with SCAL recommendationed data. \
     Path to a single file, **not** per realization/ensemble. The path can be absolute or \
-    relative to the `Webviz` configuration
+    relative to the `webviz` configuration.
 * **`sheet_name`:** Which sheet to use for the `scalfile`, only relevant if `scalfile` is an \
-    `xlsx` file (recommended to use csv files with `Webviz`).
+    `xlsx` file (recommended to use csv files with `webviz`).
 
 ---
 The minimum requirement is to define `ensembles`.
@@ -45,14 +46,18 @@ recommended unless you have a very simple model/data deck.
 * One column **per** relative permeability curve (e.g. `KRW`, `KROW` and `KRG`)
 * One column **per** capillary pressure curve (e.g. `PCOW`).
 
-The file can e.g. be dumped to disc per realization by a forward model in ERT using
-`ecl2df` or `pyscal`.
+The `relpermfile` file can e.g. be dumped to disk per realization by a forward model in ERT that
+wraps the command `ecl2csv satfunc input_file -o output_file` (requires that you have `ecl2df`
+installed). A typical example could be:
+`ecl2csv satfunc eclipse/include/props/relperm.inc -o share/results/tables/relperm.csv`.
+[Link to ecl2csv satfunc documentation.](https://equinor.github.io/ecl2df/scripts.html#satfunc)
+
 
 `scalfile` is a path to __a single file of SCAL recommendations__ (for all
 realizations/ensembles). The file has to be compatible with
 [pyscal's](https://equinor.github.io/pyscal/pyscal.html#pyscal.\
 factory.PyscalFactory.load_relperm_df) input format. Including this file adds reference cases
-`Pess`, `Base` and `Opt` to the plots.
+`Pess`, `Base` and `Opt` to the plots. This file is typically a result of a SCAL study.
 
 `sheet_name` defines the sheet to use in the `scalfile`. Only relevant if `scalfile` is an
 `xlsx` file (it is recommended to use `csv` and not `xlsx` for `Webviz`).
@@ -243,7 +248,10 @@ webviz-subsurface-testdata/blob/master/reek_history_match/share/scal/scalreek.cs
                 "id": self.uuid("color_by_selector"),
                 "content": ("Choose basis for your colormap."),
             },
-            {"id": self.uuid("ensemble_selector"), "content": ("Select ensembles."),},
+            {
+                "id": self.uuid("ensemble_selector"),
+                "content": ("Select ensembles."),
+            },
             {
                 "id": self.uuid("curve_selector"),
                 "content": (
@@ -298,16 +306,22 @@ webviz-subsurface-testdata/blob/master/reek_history_match/share/scal/scalreek.cs
                             id=self.uuid("sataxis_selector"),
                             children=[
                                 html.Span(
-                                    "Saturation axis:", style={"font-weight": "bold"},
+                                    "Saturation axis:",
+                                    style={"font-weight": "bold"},
                                 ),
                                 dcc.Dropdown(
                                     id=self.uuid("sataxis"),
                                     clearable=False,
                                     options=[
-                                        {"label": i.lower().capitalize(), "value": i,}
+                                        {
+                                            "label": i.lower().capitalize(),
+                                            "value": i,
+                                        }
                                         for i in self.sat_axes
                                     ],
                                     value=self.sat_axes[0],
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                             ],
                         ),
@@ -319,17 +333,26 @@ webviz-subsurface-testdata/blob/master/reek_history_match/share/scal/scalreek.cs
                                     id=self.uuid("color_by"),
                                     clearable=False,
                                     options=[
-                                        {"label": i.lower().capitalize(), "value": i,}
+                                        {
+                                            "label": i.lower().capitalize(),
+                                            "value": i,
+                                        }
                                         for i in self.color_options
                                     ],
                                     value=self.color_options[0],
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                             ],
                         ),
                         html.Label(
                             id=self.uuid("ensemble_selector"),
                             children=[
-                                dcc.Store(id=self.uuid("stored_ensemble"), data={}),
+                                dcc.Store(
+                                    id=self.uuid("stored_ensemble"),
+                                    storage_type="session",
+                                    data={},
+                                ),
                                 html.Span("Ensembles:", style={"font-weight": "bold"}),
                                 dcc.Dropdown(
                                     id=self.uuid("ensemble"),
@@ -339,6 +362,8 @@ webviz-subsurface-testdata/blob/master/reek_history_match/share/scal/scalreek.cs
                                         {"label": i, "value": i} for i in self.ensembles
                                     ],
                                     value=self.ensembles[0],
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                             ],
                         ),
@@ -347,11 +372,19 @@ webviz-subsurface-testdata/blob/master/reek_history_match/share/scal/scalreek.cs
                             children=[
                                 html.Span("Curves:", style={"font-weight": "bold"}),
                                 dcc.Dropdown(
-                                    id=self.uuid("curve"), clearable=False, multi=True,
+                                    id=self.uuid("curve"),
+                                    clearable=False,
+                                    multi=True,
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                             ],
                         ),
-                        dcc.Store(id=self.uuid("stored_satnum"), data={}),
+                        dcc.Store(
+                            id=self.uuid("stored_satnum"),
+                            storage_type="session",
+                            data={},
+                        ),
                         html.Label(
                             id=self.uuid("satnum_selector"),
                             children=[
@@ -364,6 +397,8 @@ webviz-subsurface-testdata/blob/master/reek_history_match/share/scal/scalreek.cs
                                         {"label": i, "value": i} for i in self.satnums
                                     ],
                                     value=self.satnums[0],
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                             ],
                         ),
@@ -386,6 +421,8 @@ webviz-subsurface-testdata/blob/master/reek_history_match/share/scal/scalreek.cs
                                         },
                                     ],
                                     value="statistics",
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                             ],
                         ),
@@ -396,11 +433,19 @@ webviz-subsurface-testdata/blob/master/reek_history_match/share/scal/scalreek.cs
                                 dcc.RadioItems(
                                     id=self.uuid("linlog"),
                                     options=[
-                                        {"label": "Linear", "value": "linear",},
-                                        {"label": "Log", "value": "log",},
+                                        {
+                                            "label": "Linear",
+                                            "value": "linear",
+                                        },
+                                        {
+                                            "label": "Log",
+                                            "value": "log",
+                                        },
                                     ],
                                     value="linear",
                                     labelStyle={"display": "inline-block"},
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                             ],
                         ),
@@ -417,11 +462,16 @@ webviz-subsurface-testdata/blob/master/reek_history_match/share/scal/scalreek.cs
                                 dcc.Checklist(
                                     id=self.uuid("scal"),
                                     options=[
-                                        {"label": "Show SCAL", "value": "show_scal",},
+                                        {
+                                            "label": "Show SCAL",
+                                            "value": "show_scal",
+                                        },
                                     ],
                                     value=["show_scal"]
                                     if self.scal is not None
                                     else [],
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                             ],
                         ),
@@ -547,7 +597,13 @@ webviz-subsurface-testdata/blob/master/reek_history_match/share/scal/scalreek.cs
             """
             return (
                 self.sat_axes_maps[sataxis],
-                [{"label": i, "value": i,} for i in self.sat_axes_maps[sataxis]],
+                [
+                    {
+                        "label": i,
+                        "value": i,
+                    }
+                    for i in self.sat_axes_maps[sataxis]
+                ],
             )
 
     def add_webvizstore(self):
@@ -697,7 +753,9 @@ def add_scal_traces(df, curves, sataxis, nplots):
                     ),
                     "name": "SCAL",
                     "legendgroup": "SCAL",
-                    "marker": {"color": "black",},
+                    "marker": {
+                        "color": "black",
+                    },
                     "line": {"dash": "dash"},
                     "showlegend": curve_no == 0 and satnum_no == 0 and case_no == 0,
                 }
@@ -774,8 +832,8 @@ def add_fanchart_traces(
 ):
     """Renders a fanchart"""
 
-    fill_color = hex_to_rgb(color, 0.3)
-    line_color = hex_to_rgb(color, 1)
+    fill_color = hex_to_rgba(color, 0.3)
+    line_color = hex_to_rgba(color, 1)
     return [
         {
             "name": legend_group,
@@ -848,15 +906,6 @@ def add_fanchart_traces(
     ]
 
 
-def hex_to_rgb(hex_string, opacity=1):
-    """Converts a hex color to rgb"""
-    hex_string = hex_string.lstrip("#")
-    hlen = len(hex_string)
-    rgb = [int(hex_string[i : i + hlen // 3], 16) for i in range(0, hlen, hlen // 3)]
-    rgb.append(opacity)
-    return f"rgba{tuple(rgb)}"
-
-
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
 def plot_layout(nplots, curves, sataxis, color_by, linlog, theme):
     """
