@@ -1,5 +1,7 @@
+from typing import Optional, List, Dict, Tuple, Callable, Any, Union
 from pathlib import Path
 
+import dash
 import numpy as np
 import pandas as pd
 import dash_html_components as html
@@ -42,19 +44,20 @@ Several visualizations are available:
 
 * **`ensembles`**: Which ensembles in `shared_settings` to visualize.
 
-In addition, you need to have the following files in your realizations stored at the local path \
-`share/results/tables`:
-
-* **`rft.csv`**: A csv file containing simulated RFT data extracted from ECLIPSE RFT output files \
-using [ecl2df](https://equinor.github.io/ecl2df/ecl2df.html#module-ecl2df.rft) \
-[(example file)](https://github.com/equinor/webviz-subsurface-testdata/blob/master/\
-reek_history_match/realization-0/iter-0/share/results/tables/rft.csv).
+In addition, you need to have rft-files in your realizations stored at the local path \
+`share/results/tables`. The `rft_ert.csv` is required as input, while the `rft.csv` is optional:
 
 * **`rft_ert.csv`**: A csv file containing simulated and observed RFT data for RFT observations \
 defined in ERT \
 [(example file)](https://github.com/equinor/webviz-subsurface-testdata/blob/master/\
 reek_history_match/realization-0/iter-0/share/results/tables/rft_ert.csv).
 
+* **`rft.csv`**: A csv file containing simulated RFT data extracted from ECLIPSE RFT output files \
+using [ecl2df](https://equinor.github.io/ecl2df/ecl2df.html#module-ecl2df.rft) \
+[(example file)](https://github.com/equinor/webviz-subsurface-testdata/blob/master/\
+reek_history_match/realization-0/iter-0/share/results/tables/rft.csv). \
+Simulated RFT data can be visualized along MD if a "CONMD" column is present in \
+the dataframe and only for wells where each RFT datapoint has a unique MD.
 
 **Using aggregated data**
 
@@ -90,45 +93,44 @@ reek_history_match/share/results/polygons/faultpolygons.csv).
 
 ?> Only RFT observations marked as active in ERT are used to generate plots.
 
-?> Only TVD values are supported, plan to support MD values in a later release.
-
-The `rft_ert.csv` file currently lacks a standardized method of generation. A \
-**temporary** script can be found [here]\
-(https://github.com/equinor/webviz-subsurface-testdata/\
-blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
+The `rft_ert.csv` file can be generated by running the "MERGE_RFT_ERTOBS" forward model in ERT, \
+this will collect ERT RFT observations and merge with CSV output from the "GENDATA_RFT" forward \
+model. [ERT docs](https://fmu-docs.equinor.com/docs/ert/reference/\
+forward_models.html?highlight=gendata_rft#MERGE_RFT_ERTOBS).
 
 """
 
     def __init__(
         self,
-        app,
+        app: dash.Dash,
         csvfile_rft: Path = None,
         csvfile_rft_ert: Path = None,
-        ensembles: list = None,
+        ensembles: Optional[List[str]] = None,
         formations: Path = None,
         obsdata: Path = None,
         faultlines: Path = None,
-    ):
+    ) -> None:
         super().__init__()
         self.formations = formations
         self.faultlines = faultlines
         self.obsdata = obsdata
         self.csvfile_rft = csvfile_rft
         self.csvfile_rft_ert = csvfile_rft_ert
+
+        self.simdf = read_csv(self.csvfile_rft) if csvfile_rft is not None else None
         self.formationdf = read_csv(self.formations) if self.formations else None
         self.faultlinesdf = read_csv(self.faultlines) if self.faultlines else None
         self.obsdatadf = read_csv(self.obsdata) if self.obsdata else None
 
-        if csvfile_rft and ensembles:
+        if csvfile_rft_ert and ensembles:
             raise ValueError(
-                'Incorrent arguments. Either provide a "csvfile_rft" and "csvfile_rft_ert" or '
-                '"ensembles"'
+                'Incorrent arguments. Either provide a "csvfile_rft_ert" or "ensembles"'
             )
 
-        if csvfile_rft and csvfile_rft_ert:
-            self.simdf = read_csv(self.csvfile_rft)
+        if csvfile_rft_ert is not None:
             self.ertdatadf = read_csv(self.csvfile_rft_ert)
-        elif ensembles:
+
+        if ensembles:
             self.ens_paths = (
                 {
                     ens: app.webviz_settings["shared_settings"]["scratch_ensembles"][
@@ -142,10 +144,8 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
 
             try:
                 self.simdf = load_csv(self.ens_paths, "share/results/tables/rft.csv")
-            except KeyError as exc:
-                raise KeyError(
-                    "Csv file for Eclipse RFT output (share/results/tables/rft.csv) not found!"
-                ) from exc
+            except (KeyError, OSError):
+                self.simdf = None
 
             try:
                 self.ertdatadf = load_csv(
@@ -153,49 +153,53 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
                 )
             except KeyError as exc:
                 raise KeyError(
-                    "Csv file for ERT RFT observations/simulations "
+                    "CSV file for ERT RFT observations/simulations "
                     "(share/results/tables/rft_ert.csv) not found!"
                 ) from exc
 
-        else:
-            raise ValueError(
-                'Incorrent arguments. Either provide a "csvfile_rft" and "csvfile_rft_ert" or '
-                '"ensembles"'
-            )
         self.ertdatadf = self.ertdatadf.rename(
             columns={
                 "time": "DATE",
+                "is_active": "ACTIVE",
                 "isactive": "ACTIVE",
                 "well": "WELL",
                 "zone": "ZONE",
                 "pressure": "SIMULATED",
                 "true_vertical_depth": "TVD",
-                "obs": "OBS",
-                "error": "OBS_ERR",
+                "measured_depth": "MD",
+                "observed": "OBSERVED",
+                "obs": "OBSERVED",
+                "error": "OBSERVED_ERR",
                 "utm_x": "EAST",
                 "utm_y": "NORTH",
             }
         )
-        self.ertdatadf["DIFF"] = self.ertdatadf["SIMULATED"] - self.ertdatadf["OBS"]
+        self.ertdatadf["DIFF"] = (
+            self.ertdatadf["SIMULATED"] - self.ertdatadf["OBSERVED"]
+        )
         self.ertdatadf["ABSDIFF"] = abs(
-            self.ertdatadf["SIMULATED"] - self.ertdatadf["OBS"]
+            self.ertdatadf["SIMULATED"] - self.ertdatadf["OBSERVED"]
         )
-
         self.ertdatadf["YEAR"] = pd.to_datetime(self.ertdatadf["DATE"]).dt.year
+        self.ertdatadf = self.ertdatadf.sort_values(by="DATE")
         self.ertdatadf["DATE_IDX"] = self.ertdatadf["DATE"].apply(
             lambda x: list(self.ertdatadf["DATE"].unique()).index(x)
         )
-
-        self.ertdatadf = filter_frame(self.ertdatadf, {"ACTIVE": 1,})
+        self.date_marks = self.set_date_marks()
+        self.ertdatadf = filter_frame(
+            self.ertdatadf,
+            {
+                "ACTIVE": 1,
+            },
+        )
         self.ertdatadf["STDDEV"] = self.ertdatadf.groupby(
             ["WELL", "DATE", "ZONE", "ENSEMBLE", "TVD"]
         )["SIMULATED"].transform("std")
 
         self.set_callbacks(app)
 
-    def add_webvizstore(self):
-
-        functions = [
+    def add_webvizstore(self) -> List[Tuple[Callable, List[Dict[str, Any]]]]:
+        functions: List[Tuple[Callable, List[Dict[str, Any]]]] = [
             (
                 read_csv,
                 [
@@ -211,15 +215,11 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
                 ],
             )
         ]
-        if self.csvfile_rft is None:
+        if self.csvfile_rft_ert is None:
             functions.append(
                 (
                     load_csv,
                     [
-                        {
-                            "ensemble_paths": self.ens_paths,
-                            "csv_file": "share/results/tables/rft.csv",
-                        },
                         {
                             "ensemble_paths": self.ens_paths,
                             "csv_file": "share/results/tables/rft_ert.csv",
@@ -227,44 +227,56 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
                     ],
                 )
             )
+            try:
+                load_csv(self.ens_paths, "share/results/tables/rft.csv")
+                functions.append(
+                    (
+                        load_csv,
+                        [
+                            {
+                                "ensemble_paths": self.ens_paths,
+                                "csv_file": "share/results/tables/rft.csv",
+                            },
+                        ],
+                    )
+                )
+            except KeyError:
+                pass
+
         return functions
 
     @property
-    def well_names(self):
+    def well_names(self) -> List[str]:
         return sorted(list(self.ertdatadf["WELL"].unique()))
 
     @property
-    def zone_names(self):
+    def zone_names(self) -> List[str]:
         return sorted(list(self.ertdatadf["ZONE"].unique()))
 
     @property
-    def dates(self):
+    def dates(self) -> List[str]:
         return sorted(list(self.ertdatadf["DATE"].unique()))
 
-    def date_in_well(self, well):
+    def date_in_well(self, well: str) -> List[str]:
         df = self.ertdatadf.loc[self.ertdatadf["WELL"] == well]
-
         return [str(d) for d in list(df["DATE"].unique())]
 
     @property
-    def ensembles(self):
+    def ensembles(self) -> List[str]:
         return list(self.ertdatadf["ENSEMBLE"].unique())
 
     @property
-    def enscolors(self):
+    def enscolors(self) -> dict:
         return unique_colors(self.ensembles)
 
-    def date_marks(self):
-
+    def set_date_marks(self) -> Dict[str, Dict[str, Any]]:
         marks = {}
-
         idx_steps = np.linspace(
             start=0,
             stop=self.ertdatadf["DATE_IDX"].max(),
             num=min(5, len(self.ertdatadf["DATE_IDX"].unique())),
             dtype=int,
         )
-
         date_steps = self.ertdatadf.loc[self.ertdatadf["DATE_IDX"].isin(idx_steps)][
             "DATE"
         ].unique()
@@ -272,12 +284,15 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
         for i, date_index in enumerate(idx_steps):
             marks[str(date_index)] = {
                 "label": f"{date_steps[i]}",
-                "style": {"white-space": "nowrap", "font-weight": "bold",},
+                "style": {
+                    "white-space": "nowrap",
+                    "font-weight": "bold",
+                },
             }
         return marks
 
     @property
-    def formation_plot_selectors(self):
+    def formation_plot_selectors(self) -> List[html.Div]:
         return [
             html.Div(
                 [
@@ -293,6 +308,8 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
                         value=self.ensembles[0],
                         multi=True,
                         clearable=False,
+                        persistence=True,
+                        persistence_type="session",
                     ),
                 ]
             ),
@@ -301,7 +318,10 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
                     html.Div(
                         style={"flex": 1},
                         children=[
-                            html.Label(style={"font-weight": "bold"}, children="Well",),
+                            html.Label(
+                                style={"font-weight": "bold"},
+                                children="Well",
+                            ),
                             dcc.Dropdown(
                                 id=self.uuid("well"),
                                 options=[
@@ -310,13 +330,18 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
                                 ],
                                 value=self.well_names[0],
                                 clearable=False,
+                                persistence=True,
+                                persistence_type="session",
                             ),
                         ],
                     ),
                     html.Div(
                         style={"flex": 1},
                         children=[
-                            html.Label(style={"font-weight": "bold"}, children="Date",),
+                            html.Label(
+                                style={"font-weight": "bold"},
+                                children="Date",
+                            ),
                             dcc.Dropdown(
                                 id=self.uuid("date"),
                                 options=[
@@ -325,34 +350,81 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
                                 ],
                                 clearable=False,
                                 value=self.date_in_well(self.well_names[0])[0],
+                                persistence=True,
+                                persistence_type="session",
                             ),
                         ],
                     ),
                 ]
             ),
-            html.Div(
-                [
-                    html.Label(
-                        style={"font-weight": "bold"},
-                        children="Plot simulated results as",
-                    ),
-                    dcc.RadioItems(
-                        id=self.uuid("linetype"),
-                        options=[
-                            {"label": "Realization lines", "value": "realization",},
-                            {"label": "Statistical fanchart", "value": "fanchart",},
+            wcc.FlexBox(
+                children=[
+                    html.Div(
+                        style={"flex": 1},
+                        children=[
+                            html.Label(
+                                style={"font-weight": "bold"},
+                                children="Plot simulated results as",
+                            ),
+                            dcc.RadioItems(
+                                id=self.uuid("linetype"),
+                                options=[
+                                    {
+                                        "label": "Realization lines",
+                                        "value": "realization",
+                                    },
+                                    {
+                                        "label": "Statistical fanchart",
+                                        "value": "fanchart",
+                                    },
+                                ],
+                                value="realization",
+                                labelStyle={
+                                    "display": "inline-block",
+                                    "margin": "5px",
+                                },
+                                persistence=True,
+                                persistence_type="session",
+                            ),
                         ],
-                        value="realization",
-                        labelStyle={"display": "inline-block", "margin": "5px",},
                     ),
-                ]
+                    html.Div(
+                        style={"flex": 1},
+                        children=[
+                            html.Label(
+                                style={"font-weight": "bold"},
+                                children="Depth option",
+                            ),
+                            dcc.RadioItems(
+                                id=self.uuid("depth_option"),
+                                options=[
+                                    {
+                                        "label": "TVD",
+                                        "value": "TVD",
+                                    },
+                                    {
+                                        "label": "MD",
+                                        "value": "MD",
+                                    },
+                                ],
+                                value="TVD",
+                                labelStyle={
+                                    "display": "inline-block",
+                                    "margin": "5px",
+                                },
+                                persistence=True,
+                                persistence_type="session",
+                            ),
+                        ],
+                    ),
+                ],
             ),
         ]
 
     @property
-    def map_plot_selectors(self):
+    def map_plot_selectors(self) -> List[html.Div]:
 
-        return (
+        return [
             html.Div(
                 style={"marginRight": "10px"},
                 children=[
@@ -371,6 +443,8 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
                                 ],
                                 value=list(self.ertdatadf["ENSEMBLE"].unique())[0],
                                 clearable=False,
+                                persistence=True,
+                                persistence_type="session",
                             ),
                         ],
                     ),
@@ -390,10 +464,15 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
                                                 "label": "Standard Deviation",
                                                 "value": "STDDEV",
                                             },
-                                            {"label": "Misfit", "value": "ABSDIFF",},
+                                            {
+                                                "label": "Misfit",
+                                                "value": "ABSDIFF",
+                                            },
                                         ],
                                         value="ABSDIFF",
                                         clearable=False,
+                                        persistence=True,
+                                        persistence_type="session",
                                     ),
                                 ],
                             ),
@@ -407,21 +486,32 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
                                     dcc.Dropdown(
                                         id=self.uuid("map_color"),
                                         options=[
-                                            {"label": "Misfit", "value": "ABSDIFF",},
+                                            {
+                                                "label": "Misfit",
+                                                "value": "ABSDIFF",
+                                            },
                                             {
                                                 "label": "Standard Deviation",
                                                 "value": "STDDEV",
                                             },
-                                            {"label": "Year", "value": "YEAR",},
+                                            {
+                                                "label": "Year",
+                                                "value": "YEAR",
+                                            },
                                         ],
                                         value="STDDEV",
                                         clearable=False,
+                                        persistence=True,
+                                        persistence_type="session",
                                     ),
                                 ],
                             ),
                         ]
                     ),
-                    html.Label(style={"font-weight": "bold"}, children="Date range",),
+                    html.Label(
+                        style={"font-weight": "bold"},
+                        children="Date range",
+                    ),
                     html.Div(
                         style={"width": "100%", "height": "70px"},
                         children=[
@@ -433,61 +523,94 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
                                     self.ertdatadf["DATE_IDX"].min(),
                                     self.ertdatadf["DATE_IDX"].max(),
                                 ],
-                                marks=self.date_marks(),
+                                marks=self.date_marks,
+                                persistence=True,
+                                persistence_type="session",
                             )
                         ],
                     ),
                 ],
             ),
-        )
+        ]
 
-    def filter_layout(self, tab):
+    def filter_layout(
+        self, tab: str
+    ) -> List[dash.development.base_component.Component]:
         """Layout for shared filters"""
         return [
-            html.Label(style={"font-weight": "bold"}, children=["Ensembles"],),
+            html.Label(
+                style={"font-weight": "bold"},
+                children=["Ensembles"],
+            ),
             wcc.Select(
                 size=min(4, len(self.ensembles)),
                 id=self.uuid(f"ensemble-{tab}"),
                 options=[{"label": name, "value": name} for name in self.ensembles],
                 value=self.ensembles,
                 multi=True,
+                persistence=True,
+                persistence_type="session",
+            ),
+            html.Label(
+                style={"font-weight": "bold"},
+                children=["Wells"],
             ),
-            html.Label(style={"font-weight": "bold"}, children=["Wells"],),
             wcc.Select(
                 size=min(20, len(self.well_names)),
                 id=self.uuid(f"well-{tab}"),
                 options=[{"label": name, "value": name} for name in self.well_names],
                 value=self.well_names,
                 multi=True,
+                persistence=True,
+                persistence_type="session",
+            ),
+            html.Label(
+                style={"font-weight": "bold"},
+                children=["Zones"],
             ),
-            html.Label(style={"font-weight": "bold"}, children=["Zones"],),
             wcc.Select(
                 size=min(10, len(self.zone_names)),
                 id=self.uuid(f"zone-{tab}"),
                 options=[{"label": name, "value": name} for name in self.zone_names],
                 value=self.zone_names,
                 multi=True,
+                persistence=True,
+                persistence_type="session",
+            ),
+            html.Label(
+                style={"font-weight": "bold"},
+                children=["Dates"],
             ),
-            html.Label(style={"font-weight": "bold"}, children=["Dates"],),
             wcc.Select(
                 size=min(10, len(self.dates)),
                 id=self.uuid(f"date-{tab}"),
                 options=[{"label": name, "value": name} for name in self.dates],
                 value=self.dates,
                 multi=True,
+                persistence=True,
+                persistence_type="session",
             ),
         ]
 
-    def size_color_layout(self):
+    def size_color_layout(self) -> List[html.Div]:
         return [
             html.Div(
                 children=[
-                    html.Label(style={"font-weight": "bold"}, children="Color by",),
+                    html.Label(
+                        style={"font-weight": "bold"},
+                        children="Color by",
+                    ),
                     dcc.Dropdown(
                         id=self.uuid("crossplot_color"),
                         options=[
-                            {"label": "Misfit", "value": "ABSDIFF",},
-                            {"label": "Standard Deviation", "value": "STDDEV",},
+                            {
+                                "label": "Misfit",
+                                "value": "ABSDIFF",
+                            },
+                            {
+                                "label": "Standard Deviation",
+                                "value": "STDDEV",
+                            },
                         ],
                         value="STDDEV",
                         clearable=False,
@@ -496,22 +619,33 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
             ),
             html.Div(
                 children=[
-                    html.Label(style={"font-weight": "bold"}, children="Size by",),
+                    html.Label(
+                        style={"font-weight": "bold"},
+                        children="Size by",
+                    ),
                     dcc.Dropdown(
                         id=self.uuid("crossplot_size"),
                         options=[
-                            {"label": "Standard Deviation", "value": "STDDEV",},
-                            {"label": "Misfit", "value": "ABSDIFF",},
+                            {
+                                "label": "Standard Deviation",
+                                "value": "STDDEV",
+                            },
+                            {
+                                "label": "Misfit",
+                                "value": "ABSDIFF",
+                            },
                         ],
                         value="ABSDIFF",
                         clearable=False,
+                        persistence=True,
+                        persistence_type="session",
                     ),
                 ],
             ),
         ]
 
     @property
-    def layout(self):
+    def layout(self) -> dcc.Tabs:
 
         tabs_styles = {"height": "44px", "width": "100%"}
         tab_style = {
@@ -553,7 +687,8 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
                                 wcc.FlexBox(
                                     children=[
                                         wcc.Graph(
-                                            style={"flex": 1}, id=self.uuid("map"),
+                                            style={"flex": 1},
+                                            id=self.uuid("map"),
                                         ),
                                         wcc.Graph(
                                             style={"flex": 1},
@@ -638,11 +773,14 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
             ],
         )
 
-    def set_callbacks(self, app):
+    def set_callbacks(self, app: dash.Dash) -> None:
         @app.callback(
-            Output(self.uuid("well"), "value"), [Input(self.uuid("map"), "clickData"),],
+            Output(self.uuid("well"), "value"),
+            [
+                Input(self.uuid("map"), "clickData"),
+            ],
         )
-        def _get_clicked_well(click_data):
+        def _get_clicked_well(click_data: Dict[str, List[Dict[str, Any]]]) -> str:
             if not click_data:
                 return self.well_names[0]
             for layer in click_data["points"]:
@@ -661,7 +799,9 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
                 Input(self.uuid("map_date"), "value"),
             ],
         )
-        def _update_map(ensemble, sizeby, colorby, dates):
+        def _update_map(
+            ensemble: str, sizeby: str, colorby: str, dates: List[float]
+        ) -> Dict[str, Any]:
             figure = MapFigure(self.ertdatadf, ensemble)
             if self.faultlinesdf is not None:
                 figure.add_fault_lines(self.faultlinesdf)
@@ -676,36 +816,91 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
                 Input(self.uuid("date"), "value"),
                 Input(self.uuid("ensemble"), "value"),
                 Input(self.uuid("linetype"), "value"),
+                Input(self.uuid("depth_option"), "value"),
             ],
         )
-        def _update_formation_plot(well, date, ensembles, linetype):
+        def _update_formation_plot(
+            well: str, date: str, ensembles: List[str], linetype: str, depth_option: str
+        ) -> Dict[str, Any]:
             if date not in self.date_in_well(well):
                 raise PreventUpdate
 
             figure = FormationFigure(
-                well, self.simdf, self.ertdatadf, self.enscolors, self.obsdatadf
+                well=well,
+                ertdf=self.ertdatadf,
+                enscolors=self.enscolors,
+                depth_option=depth_option,
+                date=date,
+                ensembles=ensembles,
+                simdf=self.simdf,
+                obsdf=self.obsdatadf,
             )
+
             if self.formations is not None:
                 figure.add_formation(self.formationdf)
-            if linetype == "realization":
-                figure.add_simulated_lines(date, ensembles)
-            if linetype == "fanchart":
-                figure.add_fanchart(date, ensembles)
 
-            figure.add_observed(date)
-            figure.add_ert_observed(date)
+            figure.add_simulated_lines(linetype)
+            figure.add_additional_observations()
+            figure.add_ert_observed()
 
             return {
                 "data": figure.traces,
                 "layout": figure.layout,
             }
 
+        @app.callback(
+            Output(self.uuid("linetype"), "options"),
+            Output(self.uuid("linetype"), "value"),
+            Input(self.uuid("depth_option"), "value"),
+            State(self.uuid("linetype"), "value"),
+            State(self.uuid("well"), "value"),
+            State(self.uuid("date"), "value"),
+        )
+        def _update_linetype(
+            depth_option: str,
+            current_linetype: str,
+            current_well: str,
+            current_date: str,
+        ) -> Tuple[List[Dict[str, str]], str]:
+            if self.simdf is not None:
+                df = filter_frame(
+                    self.simdf,
+                    {"WELL": current_well, "DATE": current_date},
+                )
+                if depth_option == "TVD" or (
+                    depth_option == "MD"
+                    and "CONMD" in self.simdf
+                    and len(df["CONMD"].unique()) == len(df["DEPTH"].unique())
+                ):
+
+                    return [
+                        {
+                            "label": "Realization lines",
+                            "value": "realization",
+                        },
+                        {
+                            "label": "Statistical fanchart",
+                            "value": "fanchart",
+                        },
+                    ], current_linetype
+
+            return [
+                {
+                    "label": "Realization lines",
+                    "value": "realization",
+                },
+            ], "realization"
+
         @app.callback(
             [Output(self.uuid("date"), "options"), Output(self.uuid("date"), "value")],
-            [Input(self.uuid("well"), "value"),],
+            [
+                Input(self.uuid("well"), "value"),
+            ],
             [State(self.uuid("date"), "value")],
         )
-        def _update_date(well, current_date):
+        def _update_date(
+            well: str, current_date: str
+        ) -> Tuple[List[Dict[str, str]], str]:
             dates = self.date_in_well(well)
             available_dates = [{"label": date, "value": date} for date in dates]
             date = current_date if current_date in dates else dates[0]
@@ -720,7 +915,9 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
                 Input(self.uuid("ensemble-misfitplot"), "value"),
             ],
         )
-        def _misfit_plot(wells, zones, dates, ensembles):
+        def _misfit_plot(
+            wells: List[str], zones: List[str], dates: List[str], ensembles: List[str]
+        ) -> List[wcc.Graph]:
             df = filter_frame(
                 self.ertdatadf,
                 {"WELL": wells, "ZONE": zones, "DATE": dates, "ENSEMBLE": ensembles},
@@ -738,7 +935,14 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
                 Input(self.uuid("crossplot_color"), "value"),
             ],
         )
-        def _crossplot(wells, zones, dates, ensembles, sizeby, colorby):
+        def _crossplot(
+            wells: List[str],
+            zones: List[str],
+            dates: List[str],
+            ensembles: List[str],
+            sizeby: str,
+            colorby: str,
+        ) -> List[wcc.Graph]:
             df = filter_frame(
                 self.ertdatadf,
                 {"WELL": wells, "ZONE": zones, "DATE": dates, "ENSEMBLE": ensembles},
@@ -754,8 +958,9 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
                 Input(self.uuid("ensemble-errorplot"), "value"),
             ],
         )
-        def _errorplot(wells, zones, dates, ensembles):
-
+        def _errorplot(
+            wells: List[str], zones: List[str], dates: List[str], ensembles: List[str]
+        ) -> Dict[str, Union[list, dict]]:
             df = filter_frame(
                 self.ertdatadf,
                 {"WELL": wells, "ZONE": zones, "DATE": dates, "ENSEMBLE": ensembles},
@@ -765,5 +970,5 @@ blob/b8b7f1fdd3abc505b137b587dcd9e44bbcf411c9/preprocessing_scripts/ert_rft.py).
 
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
 @webvizstore
-def read_csv(csv_file) -> pd.DataFrame:
+def read_csv(csv_file: str) -> pd.DataFrame:
     return pd.read_csv(csv_file)
