@@ -1,8 +1,9 @@
-from typing import Union, Optional
+from typing import Union, Optional, List, Tuple, Callable
 import json
 
 import pandas as pd
 import numpy as np
+import dash
 import dash_html_components as html
 import dash_core_components as dcc
 import webviz_core_components as wcc
@@ -62,7 +63,7 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
 
     def __init__(
         self,
-        app,
+        app: dash.Dash,
         ensembles: list,
         filter_shorter: Union[int, float] = 10,
         status_file: str = "status.json",
@@ -83,7 +84,7 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
             filter_file=None,
         )
         all_data_df = make_status_df(
-            self.ens_paths, self.status_file, self.parameter_df
+            self.ens_paths, self.status_file
         )  # Has to be stored in one df due to webvizstore, see issue #206 in webviz-config
         self.job_status_df = all_data_df.loc["job"]
         self.real_status_df = all_data_df.loc["real"]
@@ -93,7 +94,7 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
         self.set_callbacks(app)
 
     @property
-    def tour_steps(self):
+    def tour_steps(self) -> List[dict]:
         return [
             {
                 "id": self.uuid("mode"),
@@ -112,7 +113,7 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
         ]
 
     @property
-    def parameters(self):
+    def parameters(self) -> List[str]:
         """Returns numerical input parameters"""
         return list(
             self.parameter_df.drop(["ENSEMBLE", "REAL"], axis=1)
@@ -122,9 +123,12 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
         )
 
     @property
-    def plot_fig(self):
+    def plot_fig(self) -> html.Div:
         return html.Div(
-            style={"overflowX": "hidden", "width": "100%",},
+            style={
+                "overflowX": "hidden",
+                "width": "100%",
+            },
             children=[
                 html.Div(
                     id=self.uuid("plot_fig"),
@@ -139,16 +143,20 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
         )
 
     @property
-    def control_div(self):
+    def control_div(self) -> html.Div:
         return html.Div(
-            style={"padding-right": 15,},
+            style={
+                "padding-right": 15,
+            },
             children=[
                 html.Label(
                     children=[
                         html.Span("Mode:", style={"font-weight": "bold"}),
                         dcc.RadioItems(
                             id=self.uuid("mode"),
-                            style={"padding-bottom": 10,},
+                            style={
+                                "padding-bottom": 10,
+                            },
                             options=[
                                 {
                                     "label": "Running time matrix",
@@ -160,6 +168,8 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
                                 },
                             ],
                             value="running_time_matrix",
+                            persistence=True,
+                            persistence_type="session",
                         ),
                     ],
                 ),
@@ -167,7 +177,9 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
                     children=[
                         html.Span("Set ensemble:", style={"font-weight": "bold"}),
                         html.Div(
-                            style={"padding-bottom": 10,},
+                            style={
+                                "padding-bottom": 10,
+                            },
                             children=[
                                 dcc.Dropdown(
                                     id=self.uuid("ensemble"),
@@ -177,6 +189,8 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
                                     ],
                                     value=self.ensembles[0],
                                     clearable=False,
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                             ],
                         ),
@@ -191,7 +205,9 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
                             style={"font-weight": "bold"},
                         ),
                         html.Div(
-                            style={"padding-bottom": 10,},
+                            style={
+                                "padding-bottom": 10,
+                            },
                             children=[
                                 dcc.Dropdown(
                                     id=self.uuid("relative_runtime"),
@@ -203,6 +219,8 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
                                         0
                                     ],
                                     clearable=False,
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                             ],
                         ),
@@ -217,7 +235,9 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
                             style={"font-weight": "bold"},
                         ),
                         html.Div(
-                            style={"padding-bottom": 10,},
+                            style={
+                                "padding-bottom": 10,
+                            },
                             children=[
                                 dcc.Dropdown(
                                     id=self.uuid("relative_real"),
@@ -229,6 +249,8 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
                                         0
                                     ],
                                     clearable=False,
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                             ],
                         ),
@@ -239,7 +261,8 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
                     style={"display": "none"},
                     children=[
                         html.Span(
-                            "Selected parameters:", style={"font-weight": "bold"},
+                            "Selected parameters:",
+                            style={"font-weight": "bold"},
                         ),
                         wcc.Select(
                             id=self.uuid("parameters"),
@@ -251,6 +274,8 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
                             multi=True,
                             value=self.visual_parameters,
                             size=min(50, len(self.visual_parameters)),
+                            persistence=True,
+                            persistence_type="session",
                         ),
                     ],
                 ),
@@ -269,6 +294,8 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
                                 },
                             ],
                             value=["filter_short"],
+                            persistence=True,
+                            persistence_type="session",
                         ),
                     ],
                 ),
@@ -276,16 +303,18 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
         )
 
     @property
-    def layout(self):
+    def layout(self) -> wcc.FlexBox:
         return wcc.FlexBox(
-            style={"padding-top": 10,},
+            style={
+                "padding-top": 10,
+            },
             children=[
                 html.Div(style={"flex": "1"}, children=self.control_div),
                 html.Div(style={"flex": "3"}, children=self.plot_fig),
             ],
         )
 
-    def set_callbacks(self, app):
+    def set_callbacks(self, app: dash.Dash) -> None:
         @app.callback(
             Output(self.uuid("fig"), "figure"),
             [
@@ -297,7 +326,14 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
                 Input(self.uuid("filter_short"), "value"),
             ],
         )
-        def _update_fig(ens, mode, rel_runtime, rel_real, params, filter_short):
+        def _update_fig(
+            ens: str,
+            mode: str,
+            rel_runtime: str,
+            rel_real: str,
+            params: Union[str, List[str]],
+            filter_short: List[str],
+        ) -> dict:
             """Update main figure
             Dependent on `mode` it will call rendering of the chosen form of visualization
             """
@@ -321,6 +357,7 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
             # Ensure selected parameters is a list
             params = params if isinstance(params, list) else [params]
             # Color by success or runtime, for runtime drop unsuccesful
+            colormap_labels: Union[List[str], None]
             if rel_real == "Successful/failed realization":
                 plot_df = self.real_status_df[self.real_status_df["ENSEMBLE"] == ens]
                 colormap = make_colormap(
@@ -356,7 +393,7 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
             ],
             [Input(self.uuid("mode"), "value")],
         )
-        def _update_mode(mode):
+        def _update_mode(mode: str) -> Tuple[dict, dict, dict, dict]:
             """Switch displayed mode between running time matrix and parallel coordinates"""
             if mode == "running_time_matrix":
                 style = (
@@ -374,7 +411,7 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
                 )
             return style
 
-    def add_webvizstore(self):
+    def add_webvizstore(self) -> List[Tuple[Callable, list]]:
         return [
             (
                 make_status_df,
@@ -382,19 +419,23 @@ blob/master/reek_history_match/realization-0/iter-0/status.json).
                     {
                         "ens_paths": self.ens_paths,
                         "status_file": self.status_file,
-                        "parameter_df": self.parameter_df,
                     }
                 ],
             ),
             (
                 load_parameters,
-                [{"ensemble_paths": self.ens_paths, "filter_file": None,},],
+                [
+                    {
+                        "ensemble_paths": self.ens_paths,
+                        "filter_file": None,
+                    },
+                ],
             ),
         ]
 
 
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
-def render_matrix(status_df, rel, theme):
+def render_matrix(status_df: pd.DataFrame, rel: str, theme: dict) -> dict:
     """Render matrix
     Returns figure object as heatmap for the chosen ensemble and scaling method.
     """
@@ -415,8 +456,16 @@ def render_matrix(status_df, rel, theme):
         "hoverinfo": "text",
         "colorscale": theme["layout"]["colorscale"]["sequential"],
         "colorbar": {
-            "tickvals": [0, 0.5, 1,],
-            "ticktext": ["0 %", "50 %", "100 %",],
+            "tickvals": [
+                0,
+                0.5,
+                1,
+            ],
+            "ticktext": [
+                "0 %",
+                "50 %",
+                "100 %",
+            ],
             "xanchor": "left",
         },
     }
@@ -426,7 +475,11 @@ def render_matrix(status_df, rel, theme):
         {
             "paper_bgcolor": "rgba(0,0,0,0)",
             "plot_bgcolor": "rgba(0,0,0,0)",
-            "margin": {"t": 50, "b": 50, "l": 50,},
+            "margin": {
+                "t": 50,
+                "b": 50,
+                "l": 50,
+            },
             "xaxis": {
                 "ticks": "",
                 "title": "Realizations",
@@ -453,7 +506,14 @@ def render_matrix(status_df, rel, theme):
 
 
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
-def render_parcoord(plot_df, params, theme, colormap, color_col, colormap_labels=None):
+def render_parcoord(
+    plot_df: pd.DataFrame,
+    params: List[str],
+    theme: dict,
+    colormap: Union[List[str], List[list]],
+    color_col: str,
+    colormap_labels: Union[List[str], None] = None,
+) -> dict:
     """Renders parallel coordinates plot"""
     # Create parcoords dimensions (one per parameter)
     dimensions = [
@@ -461,7 +521,7 @@ def render_parcoord(plot_df, params, theme, colormap, color_col, colormap_labels
     ]
 
     # Parcoords data dict
-    data = {
+    data: dict = {
         "line": {
             "color": plot_df[color_col].values.tolist(),
             "colorscale": colormap,
@@ -489,7 +549,13 @@ def render_parcoord(plot_df, params, theme, colormap, color_col, colormap_labels
         )
     else:
         data["line"].update(  # pylint: disable=expression-not-assigned
-            {"colorbar": {"title": "Running time", "xanchor": "right", "x": -0.02,},},
+            {
+                "colorbar": {
+                    "title": "Running time",
+                    "xanchor": "right",
+                    "x": -0.02,
+                },
+            },
         )
 
     layout = {}
@@ -504,7 +570,11 @@ def render_parcoord(plot_df, params, theme, colormap, color_col, colormap_labels
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
 @webvizstore
 # pylint: disable=too-many-locals
-def make_status_df(ens_paths, status_file, parameter_df) -> pd.DataFrame:
+# pylint: disable=too-many-statements
+def make_status_df(
+    ens_paths: dict,
+    status_file: str,
+) -> pd.DataFrame:
     """Return DataFrame of information from status.json files.
     *Finds status.json filepaths.
     For jobs:
@@ -515,6 +585,11 @@ def make_status_df(ens_paths, status_file, parameter_df) -> pd.DataFrame:
     *Creates DataFrame of success/failure and total running time.
     """
 
+    parameter_df = load_parameters(
+        ensemble_paths=ens_paths,
+        ensemble_set_name="EnsembleSet",
+        filter_file=None,
+    )
     # sub-method to process ensemble data when all realizations in ensemble have been processed
     def ensemble_post_processing() -> list:
         # add missing realizations to get whitespace in heatmap matrix
@@ -554,10 +629,12 @@ def make_status_df(ens_paths, status_file, parameter_df) -> pd.DataFrame:
         ]
     )
     # Initial values for local variables
-    (job_status_dfs, ens_dfs, real_status) = ([], [], [])
+    job_status_dfs: list = []
+    ens_dfs: list = []
+    real_status: list = []
     ens_max_job_runtime = 1
     ens = ""
-    reals = []
+    reals: list = []
 
     # Loop through identified filepaths and get realization data
     for row in df.itertuples(index=False):
@@ -651,7 +728,7 @@ def make_status_df(ens_paths, status_file, parameter_df) -> pd.DataFrame:
 
 
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
-def make_colormap(color_array, discrete=None):
+def make_colormap(color_array: list, discrete: int = None) -> list:
     """
     Returns a colormap:
     * If the `discrete` variable is set to an integer x, the colormap will be a discrete map of
