@@ -1,15 +1,18 @@
+from typing import List, Union, Tuple, Callable
 from pathlib import Path
 import json
 import io
+import warnings
 
 import numpy as np
+import pandas as pd
 import xtgeo
 import dash
 from dash.dependencies import Input, Output, State
 from dash.exceptions import PreventUpdate
 import dash_html_components as html
 import dash_core_components as dcc
-from webviz_subsurface_components import LayeredMap
+from webviz_subsurface_components import LeafletMap
 import webviz_core_components as wcc
 from webviz_config.webviz_store import webvizstore
 from webviz_config.common_cache import CACHE
@@ -45,10 +48,11 @@ and available for instant viewing.
     Available settings are:
     * `min`: Truncate colorscale (lower limit).
     * `max`: Truncate colorscale (upper limit).
-    * `color`: Set the colormap (default is viridis).
+    * `color`: List of hexadecimal colors.
     * `unit`: Text to display as unit in label.
 * **`wellfolder`:** Folder with RMS wells.
 * **`wellsuffix`:** File suffix for wells in well folder.
+* **`map_height`:** Set the height in pixels for the map views.
 
 ---
 The available maps are gathered from the `share/results/maps/` folder
@@ -56,7 +60,7 @@ for each realization. Subfolders are not supported.
 
 The filenames need to follow a fairly strict convention, as the filenames are used as metadata:
 `horizon_name--attribute--date` (`--date` is optional). The files should be on `irap binary`
-format (typically `.gri` or `.irapbin`) The date is of the form `YYYYMMDD` or
+format with the suffix `.gri`. The date is of the form `YYYYMMDD` or
 `YYYYMMDD_YYYYMMDD`, the latter would be for a delta surface between two dates.
 See [this folder]\
 (https://github.com/equinor/webviz-subsurface-testdata/tree/master/reek_history_match/\
@@ -73,20 +77,24 @@ attribute_settings:
     max: 10
     unit: m
   atr_b:
-    color: rainbow
+    color:
+    - "#000004"
+    - "#1b0c41"
+    - "#4a0c6b"
+    - "#781c6d"
 ```
-Valid options for `color` are `viridis` (default), `inferno`, `warm`, `cool` and `rainbow`.
 
 """
 
     def __init__(
         self,
-        app,
+        app: dash.Dash,
         ensembles: list,
         attributes: list = None,
         attribute_settings: dict = None,
         wellfolder: Path = None,
         wellsuffix: str = ".w",
+        map_height: int = 600,
     ):
 
         super().__init__()
@@ -103,11 +111,12 @@ Valid options for `color` are `viridis` (default), `inferno`, `warm`, `cool` and
             ]
             if self.surfacedf.empty:
                 raise ValueError("No surfaces found with the given attributes")
-        self.attribute_settings = attribute_settings if attribute_settings else {}
+        self.attribute_settings: dict = attribute_settings if attribute_settings else {}
+        self.map_height = map_height
         self.surfaceconfig = surfacedf_to_dict(self.surfacedf)
         self.wellfolder = wellfolder
         self.wellsuffix = wellsuffix
-        self.wellfiles = (
+        self.wellfiles: Union[List[str], None] = (
             json.load(find_files(wellfolder, wellsuffix))
             if wellfolder is not None
             else None
@@ -127,17 +136,19 @@ Valid options for `color` are `viridis` (default), `inferno`, `warm`, `cool` and
         self.set_callbacks(app)
 
     @property
-    def ensembles(self):
+    def ensembles(self) -> List[str]:
         return list(self.ens_df["ENSEMBLE"].unique())
 
-    def realizations(self, ensemble, sensname=None, senstype=None):
+    def realizations(
+        self, ensemble: str, sensname: str = None, senstype: str = None
+    ) -> List[str]:
         df = self.ens_df.loc[self.ens_df["ENSEMBLE"] == ensemble].copy()
         if sensname and senstype:
             df = df.loc[(df["SENSNAME"] == sensname) & (df["SENSCASE"] == senstype)]
         return list(df["REAL"]) + ["Mean", "StdDev", "Min", "Max"]
 
     @property
-    def tour_steps(self):
+    def tour_steps(self) -> List[dict]:
         return [
             {
                 "id": self.uuid("layout"),
@@ -162,7 +173,7 @@ Valid options for `color` are `viridis` (default), `inferno`, `warm`, `cool` and
         ]
 
     @staticmethod
-    def set_grid_layout(columns):
+    def set_grid_layout(columns: str) -> dict:
         return {
             "display": "grid",
             "alignContent": "space-around",
@@ -171,8 +182,14 @@ Valid options for `color` are `viridis` (default), `inferno`, `warm`, `cool` and
         }
 
     def ensemble_layout(
-        self, ensemble_id, ens_prev_id, ens_next_id, real_id, real_prev_id, real_next_id
-    ):
+        self,
+        ensemble_id: str,
+        ens_prev_id: str,
+        ens_next_id: str,
+        real_id: str,
+        real_prev_id: str,
+        real_next_id: str,
+    ) -> wcc.FlexBox:
         return wcc.FlexBox(
             children=[
                 html.Div(
@@ -189,6 +206,8 @@ Valid options for `color` are `viridis` (default), `inferno`, `warm`, `cool` and
                                     value=self.ensembles[0],
                                     id=ensemble_id,
                                     clearable=False,
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                                 html.Button(
                                     style={
@@ -226,6 +245,8 @@ Valid options for `color` are `viridis` (default), `inferno`, `warm`, `cool` and
                                     value=self.realizations(self.ensembles[0])[0],
                                     id=real_id,
                                     clearable=False,
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                                 html.Button(
                                     style={
@@ -253,7 +274,7 @@ Valid options for `color` are `viridis` (default), `inferno`, `warm`, `cool` and
         )
 
     @property
-    def layout(self):
+    def layout(self) -> html.Div:
         return html.Div(
             id=self.uuid("layout"),
             children=[
@@ -309,6 +330,8 @@ Valid options for `color` are `viridis` (default), `inferno`, `warm`, `cool` and
                                                 "Quotient",
                                             ]
                                         ],
+                                        persistence=True,
+                                        persistence_type="session",
                                     )
                                 ),
                                 wcc.FlexBox(
@@ -321,6 +344,8 @@ Valid options for `color` are `viridis` (default), `inferno`, `warm`, `cool` and
                                                     debounce=True,
                                                     type="number",
                                                     id=self.uuid("truncate-diff-min"),
+                                                    persistence=True,
+                                                    persistence_type="session",
                                                 ),
                                             ],
                                         ),
@@ -332,6 +357,8 @@ Valid options for `color` are `viridis` (default), `inferno`, `warm`, `cool` and
                                                     debounce=True,
                                                     type="number",
                                                     id=self.uuid("truncate-diff-max"),
+                                                    persistence=True,
+                                                    persistence_type="session",
                                                 ),
                                             ],
                                         ),
@@ -349,71 +376,184 @@ Valid options for `color` are `viridis` (default), `inferno`, `warm`, `cool` and
                     style={"fontSize": "1rem"},
                     children=[
                         html.Div(
-                            style={"margin": "10px", "flex": 4},
+                            style={
+                                "height": self.map_height,
+                                "margin": "10px",
+                                "flex": 4,
+                            },
                             children=[
-                                LayeredMap(
-                                    sync_ids=[self.uuid("map2"), self.uuid("map3")],
+                                LeafletMap(
+                                    syncedMaps=[self.uuid("map2"), self.uuid("map3")],
                                     id=self.uuid("map"),
-                                    height=600,
                                     layers=[],
-                                    hillShading=True,
-                                )
+                                    unitScale={},
+                                    autoScaleMap=True,
+                                    minZoom=-5,
+                                    updateMode="update",
+                                    mouseCoords={"position": "bottomright"},
+                                    colorBar={"position": "bottomleft"},
+                                ),
+                                html.Div(
+                                    children=[
+                                        dcc.RadioItems(
+                                            id=self.uuid("hillshade"),
+                                            labelStyle={
+                                                "display": "inline-block",
+                                                "text-align": "justify",
+                                            },
+                                            options=[
+                                                {
+                                                    "value": None,
+                                                    "label": "No hillshading",
+                                                },
+                                                {
+                                                    "value": "soft-hillshading",
+                                                    "label": "Soft hillshading",
+                                                },
+                                                {
+                                                    "value": "hillshading",
+                                                    "label": "Hillshading",
+                                                },
+                                                {
+                                                    "value": "hillshading_shadows",
+                                                    "label": "Hillshading with shadows",
+                                                },
+                                            ],
+                                            value=None,
+                                            persistence=True,
+                                            persistence_type="session",
+                                        ),
+                                    ]
+                                ),
                             ],
                         ),
                         html.Div(
                             style={"margin": "10px", "flex": 4},
                             children=[
-                                LayeredMap(
-                                    sync_ids=[self.uuid("map"), self.uuid("map3")],
+                                LeafletMap(
+                                    syncedMaps=[self.uuid("map"), self.uuid("map3")],
                                     id=self.uuid("map2"),
-                                    height=600,
                                     layers=[],
-                                    hillShading=True,
-                                )
+                                    unitScale={},
+                                    autoScaleMap=True,
+                                    minZoom=-5,
+                                    updateMode="update",
+                                    mouseCoords={"position": "bottomright"},
+                                    colorBar={"position": "bottomleft"},
+                                ),
+                                html.Div(
+                                    children=[
+                                        dcc.RadioItems(
+                                            id=self.uuid("hillshade2"),
+                                            labelStyle={
+                                                "display": "inline-block",
+                                                "text-align": "justify",
+                                            },
+                                            options=[
+                                                {
+                                                    "value": None,
+                                                    "label": "No hillshading",
+                                                },
+                                                {
+                                                    "value": "soft-hillshading",
+                                                    "label": "Soft hillshading",
+                                                },
+                                                {
+                                                    "value": "hillshading",
+                                                    "label": "Hillshading",
+                                                },
+                                                {
+                                                    "value": "hillshading_shadows",
+                                                    "label": "Hillshading with shadows",
+                                                },
+                                            ],
+                                            value=None,
+                                            persistence=True,
+                                            persistence_type="session",
+                                        ),
+                                    ]
+                                ),
                             ],
                         ),
                         html.Div(
                             style={"margin": "10px", "flex": 4},
                             children=[
-                                LayeredMap(
-                                    sync_ids=[self.uuid("map"), self.uuid("map2")],
+                                LeafletMap(
+                                    syncedMaps=[self.uuid("map"), self.uuid("map2")],
                                     id=self.uuid("map3"),
-                                    height=600,
                                     layers=[],
-                                    hillShading=True,
-                                )
+                                    unitScale={},
+                                    autoScaleMap=True,
+                                    minZoom=-5,
+                                    updateMode="update",
+                                    mouseCoords={"position": "bottomright"},
+                                    colorBar={"position": "bottomleft"},
+                                ),
+                                html.Div(
+                                    children=[
+                                        dcc.RadioItems(
+                                            id=self.uuid("hillshade3"),
+                                            labelStyle={
+                                                "display": "inline-block",
+                                                "text-align": "justify",
+                                            },
+                                            options=[
+                                                {
+                                                    "value": None,
+                                                    "label": "No hillshading",
+                                                },
+                                                {
+                                                    "value": "soft-hillshading",
+                                                    "label": "Soft hillshading",
+                                                },
+                                                {
+                                                    "value": "hillshading",
+                                                    "label": "Hillshading",
+                                                },
+                                                {
+                                                    "value": "hillshading_shadows",
+                                                    "label": "Hillshading with shadows",
+                                                },
+                                            ],
+                                            value=None,
+                                            persistence=True,
+                                            persistence_type="session",
+                                        ),
+                                    ]
+                                ),
                             ],
                         ),
                         dcc.Store(
                             id=self.uuid("attribute-settings"),
                             data=json.dumps(self.attribute_settings),
+                            storage_type="session",
                         ),
                     ],
                 ),
             ],
         )
 
-    def get_real_runpath(self, data, ensemble, real):
-        data = make_fmu_filename(data)
+    def get_real_runpath(self, data: dict, ensemble: str, real: str) -> Path:
+        filename = make_fmu_filename(data)
         runpath = Path(
             self.ens_df.loc[
                 (self.ens_df["ENSEMBLE"] == ensemble) & (self.ens_df["REAL"] == real)
             ]["RUNPATH"].unique()[0]
         )
 
-        return get_path(runpath / "share" / "results" / "maps" / f"{data}.gri")
+        return get_path(runpath / "share" / "results" / "maps" / f"{filename}.gri")
 
-    def get_ens_runpath(self, data, ensemble):
-        data = make_fmu_filename(data)
+    def get_ens_runpath(self, data: dict, ensemble: str) -> List[Path]:
+        filename = make_fmu_filename(data)
         runpaths = self.ens_df.loc[(self.ens_df["ENSEMBLE"] == ensemble)][
             "RUNPATH"
         ].unique()
         return [
-            Path(runpath) / "share" / "results" / "maps" / f"{data}.gri"
+            Path(runpath) / "share" / "results" / "maps" / f"{filename}.gri"
             for runpath in runpaths
         ]
 
-    def set_callbacks(self, app):
+    def set_callbacks(self, app: dash.Dash) -> None:
         @app.callback(
             [
                 Output(self.uuid("map"), "layers"),
@@ -432,26 +572,45 @@ Valid options for `color` are `viridis` (default), `inferno`, `warm`, `cool` and
                 Input(self.uuid("attribute-settings"), "data"),
                 Input(self.uuid("truncate-diff-min"), "value"),
                 Input(self.uuid("truncate-diff-max"), "value"),
+                Input(self.uuid("hillshade"), "value"),
+                Input(self.uuid("hillshade2"), "value"),
+                Input(self.uuid("hillshade3"), "value"),
             ],
         )
         # pylint: disable=too-many-arguments, too-many-locals
         def _set_base_layer(
-            data,
-            ensemble,
-            real,
-            data2,
-            ensemble2,
-            real2,
-            calculation,
-            attribute_settings,
-            diff_min,
-            diff_max,
-        ):
-            if not data or not data2:
+            stored_selector_data: str,
+            ensemble: str,
+            real: str,
+            stored_selector2_data: str,
+            ensemble2: str,
+            real2: str,
+            calculation: str,
+            stored_attribute_settings: str,
+            diff_min: Union[int, float, None],
+            diff_max: Union[int, float, None],
+            hillshade: Union[str, None],
+            hillshade2: Union[str, None],
+            hillshade3: Union[str, None],
+        ) -> Tuple[List[dict], List[dict], List[dict], str]:
+            if not stored_selector_data or not stored_selector2_data:
                 raise PreventUpdate
-            data = json.loads(data)
-            data2 = json.loads(data2)
-            attribute_settings = json.loads(attribute_settings)
+
+            # TODO(Sigurd)
+            # These two are presumably of type dict, but the type depends on the actual python
+            # objects that get serialized inside SurfaceSelector.
+            # Should deserialization and validation be delegated to SurfaceSelector?
+            # Note that according to the doc, it seems that dcc.Store actualy does the
+            # serialization/deserialization for us!
+            # Should be refactored
+            data: dict = json.loads(stored_selector_data)
+            data2: dict = json.loads(stored_selector2_data)
+            if not isinstance(data, dict) or not isinstance(data2, dict):
+                raise TypeError("Selector data payload must be of type dict")
+
+            attribute_settings: dict = json.loads(stored_attribute_settings)
+            if not isinstance(attribute_settings, dict):
+                raise TypeError("Expected stored attribute_settings to be of type dict")
 
             if real in ["Mean", "StdDev", "Min", "Max"]:
                 surface = calculate_surface(self.get_ens_runpath(data, ensemble), real)
@@ -466,30 +625,26 @@ Valid options for `color` are `viridis` (default), `inferno`, `warm`, `cool` and
             else:
                 surface2 = load_surface(self.get_real_runpath(data2, ensemble2, real2))
 
-            surface_layers = [
+            surface_layers: List[dict] = [
                 make_surface_layer(
                     surface,
                     name="surface",
-                    color=attribute_settings.get(data["attr"], {}).get(
-                        "color", "viridis"
-                    ),
+                    color=attribute_settings.get(data["attr"], {}).get("color"),
+                    shader_type=hillshade,
                     min_val=attribute_settings.get(data["attr"], {}).get("min", None),
                     max_val=attribute_settings.get(data["attr"], {}).get("max", None),
-                    unit=attribute_settings.get(data2["attr"], {}).get("unit", ""),
-                    hillshading=True,
+                    unit=attribute_settings.get(data["attr"], {}).get("unit", " "),
                 )
             ]
-            surface_layers2 = [
+            surface_layers2: List[dict] = [
                 make_surface_layer(
                     surface2,
-                    name="surface",
-                    color=attribute_settings.get(data2["attr"], {}).get(
-                        "color", "viridis"
-                    ),
+                    name="surface2",
+                    color=attribute_settings.get(data2["attr"], {}).get("color"),
+                    shader_type=hillshade2,
                     min_val=attribute_settings.get(data2["attr"], {}).get("min", None),
                     max_val=attribute_settings.get(data2["attr"], {}).get("max", None),
-                    unit=attribute_settings.get(data2["attr"], {}).get("unit", ""),
-                    hillshading=True,
+                    unit=attribute_settings.get(data2["attr"], {}).get("unit", " "),
                 )
             ]
 
@@ -499,15 +654,13 @@ Valid options for `color` are `viridis` (default), `inferno`, `warm`, `cool` and
                     surface3.values[surface3.values <= diff_min] = diff_min
                 if diff_max is not None:
                     surface3.values[surface3.values >= diff_max] = diff_max
-                diff_layers = []
+                diff_layers: List[dict] = []
                 diff_layers.append(
                     make_surface_layer(
                         surface3,
-                        name="surface",
-                        color=attribute_settings.get(data["attr"], {}).get(
-                            "color", "viridis"
-                        ),
-                        hillshading=True,
+                        name="surface3",
+                        color=attribute_settings.get(data["attr"], {}).get("color"),
+                        shader_type=hillshade3,
                     )
                 )
                 error_label = ""
@@ -516,15 +669,18 @@ Valid options for `color` are `viridis` (default), `inferno`, `warm`, `cool` and
                 error_label = (
                     "Cannot calculate because the surfaces have different geometries"
                 )
+
             if self.well_layer:
                 surface_layers.append(self.well_layer)
                 surface_layers2.append(self.well_layer)
                 diff_layers.append(self.well_layer)
             return (surface_layers, surface_layers2, diff_layers, error_label)
 
-        def _update_from_btn(_n_prev, _n_next, current_value, options):
+        def _update_from_btn(
+            _n_prev: int, _n_next: int, current_value: str, options: List[dict]
+        ) -> str:
             """Updates dropdown value if previous/next btn is clicked"""
-            options = [opt["value"] for opt in options]
+            option_values: List[str] = [opt["value"] for opt in options]
             ctx = dash.callback_context.triggered
             if not ctx or current_value is None:
                 raise PreventUpdate
@@ -532,9 +688,9 @@ Valid options for `color` are `viridis` (default), `inferno`, `warm`, `cool` and
                 return current_value
             callback = ctx[0]["prop_id"]
             if "-prev" in callback:
-                return prev_value(current_value, options)
+                return prev_value(current_value, option_values)
             if "-next" in callback:
-                return next_value(current_value, options)
+                return next_value(current_value, option_values)
             return current_value
 
         for btn_name in ["ensemble", "realization", "ensemble2", "realization2"]:
@@ -550,8 +706,8 @@ Valid options for `color` are `viridis` (default), `inferno`, `warm`, `cool` and
                 ],
             )(_update_from_btn)
 
-    def add_webvizstore(self):
-        store_functions = [
+    def add_webvizstore(self) -> List[Tuple[Callable, list]]:
+        store_functions: List[Tuple[Callable, list]] = [
             (
                 find_surfaces,
                 [
@@ -617,33 +773,47 @@ Valid options for `color` are `viridis` (default), `inferno`, `warm`, `cool` and
 
 
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
-def calculate_surface(fns, statistic):
+def calculate_surface(fns: List[str], statistic: str) -> xtgeo.RegularSurface:
     return surface_from_json(json.load(save_surface(fns, statistic)))
 
 
 @webvizstore
-def save_surface(fns, statistic) -> io.BytesIO:
+def save_surface(fns: List[str], statistic: str) -> io.BytesIO:
     surfaces = xtgeo.Surfaces(fns)
     if len(surfaces.surfaces) == 0:
         surface = xtgeo.RegularSurface()
-    elif statistic == "Mean":
-        surface = surfaces.apply(np.nanmean, axis=0)
-    elif statistic == "StdDev":
-        surface = surfaces.apply(np.nanstd, axis=0)
-    elif statistic == "Min":
-        surface = surfaces.apply(np.nanmin, axis=0)
-    elif statistic == "Max":
-        surface = surfaces.apply(np.nanmax, axis=0)
-    elif statistic == "P10":
-        surface = surfaces.apply(np.nanpercentile, 10, axis=0)
-    elif statistic == "P90":
-        surface = surfaces.apply(np.nanpercentile, 90, axis=0)
+    elif statistic in ["Mean", "StdDev", "Min", "Max", "P10", "P90"]:
+        # Suppress numpy warnings when surfaces have undefined z-values
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore", "All-NaN slice encountered")
+            warnings.filterwarnings("ignore", "Mean of empty slice")
+            warnings.filterwarnings("ignore", "Degrees of freedom <= 0 for slice")
+            surface = calculate_statistic(surfaces, statistic)
     else:
         surface = xtgeo.RegularSurface()
     return io.BytesIO(surface_to_json(surface).encode())
 
 
-def surface_to_json(surface):
+# pylint: disable=too-many-return-statements
+def calculate_statistic(
+    surfaces: xtgeo.Surfaces, statistic: str
+) -> xtgeo.RegularSurface:
+    if statistic == "Mean":
+        return surfaces.apply(np.nanmean, axis=0)
+    if statistic == "StdDev":
+        return surfaces.apply(np.nanstd, axis=0)
+    if statistic == "Min":
+        return surfaces.apply(np.nanmin, axis=0)
+    if statistic == "Max":
+        return surfaces.apply(np.nanmax, axis=0)
+    if statistic == "P10":
+        return surfaces.apply(np.nanpercentile, 10, axis=0)
+    if statistic == "P90":
+        return surfaces.apply(np.nanpercentile, 90, axis=0)
+    return xtgeo.RegularSurface()
+
+
+def surface_to_json(surface: xtgeo.RegularSurface) -> str:
     return json.dumps(
         {
             "ncol": surface.ncol,
@@ -658,7 +828,7 @@ def surface_to_json(surface):
     )
 
 
-def surface_from_json(surfaceobj):
+def surface_from_json(surfaceobj: dict) -> xtgeo.RegularSurface:
     # See https://github.com/equinor/xtgeo/issues/405
     surface = xtgeo.RegularSurface(**surfaceobj)
     surface.values = np.array(surfaceobj["values"])
@@ -666,16 +836,16 @@ def surface_from_json(surfaceobj):
 
 
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
-def get_surfaces(fns):
-    return xtgeo.surface.surfaces.Surfaces(fns)
+def get_surfaces(fns: List[str]) -> xtgeo.Surfaces:
+    return xtgeo.Surfaces(fns)
 
 
 @webvizstore
-def get_path(path) -> Path:
+def get_path(path: Path) -> Path:
     return Path(path)
 
 
-def prev_value(current_value, options):
+def prev_value(current_value: str, options: List[str]) -> str:
     try:
         index = options.index(current_value)
         return options[max(0, index - 1)]
@@ -683,7 +853,7 @@ def prev_value(current_value, options):
         return current_value
 
 
-def next_value(current_value, options):
+def next_value(current_value: str, options: List[str]) -> str:
     try:
         index = options.index(current_value)
         return options[min(len(options) - 1, index + 1)]
@@ -692,7 +862,7 @@ def next_value(current_value, options):
         return current_value
 
 
-def surfacedf_to_dict(df):
+def surfacedf_to_dict(df: pd.DataFrame) -> dict:
     return {
         attr: {
             "names": list(dframe["name"].unique()),
@@ -705,7 +875,7 @@ def surfacedf_to_dict(df):
 
 
 @webvizstore
-def find_files(folder, suffix) -> io.BytesIO:
+def find_files(folder: Path, suffix: str) -> io.BytesIO:
     return io.BytesIO(
         json.dumps(
             sorted([str(filename) for filename in folder.glob(f"*{suffix}")])
@@ -713,14 +883,18 @@ def find_files(folder, suffix) -> io.BytesIO:
     )
 
 
-def make_fmu_filename(data):
+def make_fmu_filename(data: dict) -> str:
     filename = f"{data['name']}--{data['attr']}"
     if data["date"] is not None:
         filename += f"--{data['date']}"
     return filename
 
 
-def calculate_surface_difference(surface, surface2, calculation="Difference"):
+def calculate_surface_difference(
+    surface: xtgeo.RegularSurface,
+    surface2: xtgeo.RegularSurface,
+    calculation: str = "Difference",
+) -> xtgeo.RegularSurface:
     surface3 = surface.copy()
     if calculation == "Difference":
         surface3.values = surface3.values - surface2.values
