@@ -1,16 +1,15 @@
+from typing import List
 from uuid import uuid4
 from pathlib import Path
-from typing import List
 
 import pandas as pd
-from matplotlib.colors import ListedColormap
 import xtgeo
 from dash.exceptions import PreventUpdate
 from dash.dependencies import Input, Output, State
 import dash_html_components as html
 import dash_core_components as dcc
 import webviz_core_components as wcc
-from webviz_subsurface_components import LayeredMap
+from webviz_subsurface_components import LeafletMap
 from webviz_config import WebvizPluginABC
 from webviz_config.webviz_store import webvizstore
 from webviz_config.utils import calculate_slider_step
@@ -125,7 +124,10 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                     "Plugin to display surfaces and random lines from a 3D grid parameter. "
                 ),
             },
-            {"id": self.ids("surface"), "content": ("The visualized surface."),},
+            {
+                "id": self.ids("surface"),
+                "content": ("The visualized surface."),
+            },
             {
                 "id": self.ids("map-view"),
                 "content": (
@@ -193,6 +195,8 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                                     ],
                                     value=self.surfacefiles[0],
                                     clearable=False,
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                             ]
                         ),
@@ -211,6 +215,8 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                                         },
                                     ],
                                     value="surface",
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                             ],
                         ),
@@ -224,12 +230,54 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                                 "height": "800px",
                                 "zIndex": -9999,
                             },
-                            children=LayeredMap(
-                                id=self.ids("map-view"),
-                                draw_toolbar_polyline=True,
-                                hillShading=True,
-                                layers=[],
-                            ),
+                            children=[
+                                LeafletMap(
+                                    id=self.ids("map-view"),
+                                    autoScaleMap=True,
+                                    minZoom=-5,
+                                    updateMode="update",
+                                    drawTools={
+                                        "drawMarker": False,
+                                        "drawPolygon": False,
+                                        "drawPolyline": True,
+                                        "position": "topright",
+                                    },
+                                    mouseCoords={"position": "bottomright"},
+                                    colorBar={"position": "bottomleft"},
+                                ),
+                                html.Div(
+                                    children=[
+                                        dcc.RadioItems(
+                                            id=self.uuid("hillshade"),
+                                            labelStyle={
+                                                "display": "inline-block",
+                                                "text-align": "justify",
+                                            },
+                                            options=[
+                                                {
+                                                    "value": None,
+                                                    "label": "No hillshading",
+                                                },
+                                                {
+                                                    "value": "soft-hillshading",
+                                                    "label": "Soft hillshading",
+                                                },
+                                                {
+                                                    "value": "hillshading",
+                                                    "label": "Hillshading",
+                                                },
+                                                {
+                                                    "value": "hillshading_shadows",
+                                                    "label": "Hillshading with shadows",
+                                                },
+                                            ],
+                                            value=None,
+                                            persistence=True,
+                                            persistence_type="session",
+                                        ),
+                                    ]
+                                ),
+                            ],
                         )
                     ]
                 ),
@@ -262,6 +310,8 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                                     ],
                                     value=self.gridparafiles[0],
                                     clearable=False,
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                             ]
                         ),
@@ -304,6 +354,8 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                                 dcc.RangeSlider(
                                     id=self.ids("color-values"),
                                     tooltip={"always_visible": True},
+                                    persistence=True,
+                                    persistence_type="session",
                                 ),
                             ],
                         ),
@@ -343,31 +395,30 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                 Input(self.ids("gridparameter"), "value"),
                 Input(self.ids("color-values"), "value"),
                 Input(self.ids("color-scale"), "colorscale"),
+                Input(self.uuid("hillshade"), "value"),
             ],
         )
         def _render_surface(
-            surfacepath, surface_type, gridparameter, color_values, colorscale
+            surfacepath,
+            surface_type,
+            gridparameter,
+            color_values,
+            colorscale,
+            hillshade,
         ):
 
             surface = xtgeo.RegularSurface(get_path(surfacepath))
-            hillshading = True
             min_val = None
             max_val = None
-            color = "viridis"
+            color = None
 
             if surface_type == "attribute":
-                hillshading = False
                 min_val = color_values[0] if color_values else None
                 max_val = color_values[1] if color_values else None
-                color = ListedColormap(colorscale) if colorscale else "viridis"
+                color = colorscale if colorscale else color
                 grid = load_grid(get_path(self.gridfile))
                 gridparameter = load_grid_parameter(grid, get_path(gridparameter))
                 surface.slice_grid3d(grid, gridparameter)
-                surface.values = surface.values.filled(0)
-                if min_val is not None:
-                    surface.values[surface.values < min_val] = min_val
-                if max_val is not None:
-                    surface.values[surface.values > max_val] = max_val
 
             s_layer = make_surface_layer(
                 surface,
@@ -375,7 +426,7 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                 min_val=min_val,
                 max_val=max_val,
                 color=color,
-                hillshading=hillshading,
+                shader_type=hillshade,
             )
             return [s_layer]
 
