@@ -1,13 +1,14 @@
+from typing import List, Dict, Union, Tuple, Any, Callable
 from uuid import uuid4
 from pathlib import Path
-from typing import List
 
+import dash
 from dash.exceptions import PreventUpdate
 from dash.dependencies import Input, Output, State
 import dash_html_components as html
 import dash_core_components as dcc
 import webviz_core_components as wcc
-from webviz_subsurface_components import LayeredMap
+from webviz_subsurface_components import LeafletMap
 from webviz_config import WebvizPluginABC
 from webviz_config.webviz_store import webvizstore
 
@@ -63,13 +64,13 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
     # pylint: disable=too-many-arguments
     def __init__(
         self,
-        app,
+        app: dash.Dash,
         surfacefiles: List[Path],
         wellfiles: List[Path],
         segyfiles: List[Path] = None,
         surfacenames: list = None,
         zonelog: str = None,
-        zunit="depth (m)",
+        zunit: str = "depth (m)",
         zmin: float = None,
         zmax: float = None,
         zonemin: int = 1,
@@ -85,23 +86,22 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
         self.zmin = zmin
         self.zmax = zmax
         self.zonemin = zonemin
-        self.surfacenames = surfacenames
         self.surfacefiles = [str(surface) for surface in surfacefiles]
         self.wellfiles = [str(well) for well in wellfiles]
         self.segyfiles = [str(segy) for segy in segyfiles] if segyfiles else []
-        self.surfacenames = (
+        self.surfacenames: List[str] = (
             surfacenames if surfacenames else [surface.stem for surface in surfacefiles]
         )
         self.zonelog = zonelog
         self.uid = uuid4()
         self.set_callbacks(app)
 
-    def ids(self, element):
+    def ids(self, element: str) -> str:
         """Generate unique id for dom element"""
         return f"{element}-id-{self.uid}"
 
     @property
-    def well_layout(self):
+    def well_layout(self) -> html.Div:
         return html.Div(
             children=html.Label(
                 children=[
@@ -114,13 +114,15 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                         ],
                         value=self.wellfiles[0],
                         clearable=False,
+                        persistence=True,
+                        persistence_type="session",
                     ),
                 ]
             ),
         )
 
     @property
-    def seismic_layout(self):
+    def seismic_layout(self) -> html.Div:
         return html.Div(
             style={} if self.segyfiles else {"display": "none"},
             children=html.Label(
@@ -136,13 +138,15 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                         else None,
                         value=self.segyfiles[0] if self.segyfiles else None,
                         clearable=False,
+                        persistence=True,
+                        persistence_type="session",
                     ),
                 ]
             ),
         )
 
     @property
-    def viz_options_layout(self):
+    def viz_options_layout(self) -> dcc.Checklist:
         options = [{"label": "Show surface fill", "value": "show_surface_fill"}]
         value = ["show_surface_fill"]
         if self.segyfiles:
@@ -151,10 +155,16 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
             options.append({"label": "Show zonelog", "value": "show_zonelog"})
             value.append("show_zonelog")
 
-        return dcc.Checklist(id=self.ids("options"), options=options, value=value,)
+        return dcc.Checklist(
+            id=self.ids("options"),
+            options=options,
+            value=value,
+            persistence=True,
+            persistence_type="session",
+        )
 
     @property
-    def well_options(self):
+    def well_options(self) -> html.Div:
         return html.Div(
             style={"marginLeft": "20px", "marginRight": "0px", "marginBotton": "0px"},
             children=[
@@ -166,7 +176,10 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                                 id=self.ids("sampling"),
                                 debounce=True,
                                 type="number",
+                                required=True,
                                 value=self.sampling,
+                                persistence=True,
+                                persistence_type="session",
                             ),
                         ]
                     )
@@ -179,7 +192,10 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                                 id=self.ids("nextend"),
                                 debounce=True,
                                 type="number",
+                                required=True,
                                 value=self.nextend,
+                                persistence=True,
+                                persistence_type="session",
                             ),
                         ]
                     )
@@ -188,7 +204,7 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
         )
 
     @property
-    def layout(self):
+    def layout(self) -> html.Div:
         return html.Div(
             children=[
                 html.Div(
@@ -198,7 +214,10 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                         self.well_options,
                         self.seismic_layout,
                         self.viz_options_layout,
-                        html.Button(id=self.ids("show_map"), children="Show map",),
+                        html.Button(
+                            id=self.ids("show_map"),
+                            children="Show map",
+                        ),
                     ],
                 ),
                 html.Div(
@@ -215,8 +234,15 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                                 "zIndex": 10000,
                                 "visibility": "hidden",
                             },
-                            children=LayeredMap(
-                                height=400, id=self.ids("map"), layers=[]
+                            children=LeafletMap(
+                                id=self.ids("map"),
+                                layers=[],
+                                unitScale={},
+                                autoScaleMap=True,
+                                minZoom=-5,
+                                updateMode="update",
+                                mouseCoords={"position": "bottomright"},
+                                colorBar={"position": "bottomleft"},
                             ),
                         ),
                         wcc.Graph(id=self.ids("graph")),
@@ -226,7 +252,7 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
         )
 
     @staticmethod
-    def set_grid_layout(columns):
+    def set_grid_layout(columns: str) -> Dict[str, str]:
         return {
             "display": "grid",
             "alignContent": "space-around",
@@ -234,7 +260,7 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
             "gridTemplateColumns": f"{columns}",
         }
 
-    def set_callbacks(self, app):
+    def set_callbacks(self, app: dash.Dash) -> None:
         @app.callback(
             Output(self.ids("graph"), "figure"),
             [
@@ -245,9 +271,19 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                 Input(self.ids("nextend"), "value"),
             ],
         )
-        def _render_section(well, cube, options, sampling, nextend):
+        def _render_section(
+            wellfile: str,
+            cubefile: str,
+            options: List[str],
+            sampling: Union[int, float, None],
+            nextend: Union[int, float, None],
+        ) -> dict:
             """Update cross section"""
-            well = load_well(get_path(well))
+
+            if sampling is None or nextend is None:
+                raise PreventUpdate
+
+            well = load_well(get_path(wellfile))
             xsect = XSectionFigure(
                 well=well,
                 zmin=self.zmin,
@@ -265,7 +301,7 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
             )
 
             if "show_seismic" in options:
-                cube = load_cube_data(get_path(cube))
+                cube = load_cube_data(get_path(cubefile))
                 xsect.plot_cube(cube)
 
             xsect.plot_well(
@@ -283,7 +319,7 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
             [Input(self.ids("show_map"), "n_clicks")],
             [State(self.ids("map_wrapper"), "style")],
         )
-        def _show_map(nclicks, style):
+        def _show_map(nclicks: int, style: dict) -> Tuple[dict, str]:
             """Show/hide map on button click"""
             btn = "Show Map"
             if not nclicks:
@@ -296,22 +332,23 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
             return style, btn
 
         @app.callback(
-            [Output(self.ids("map"), "layers"), Output(self.ids("map"), "uirevision")],
+            Output(self.ids("map"), "layers"),
             [Input(self.ids("wells"), "value")],
         )
-        def _render_surface(wellfile):
+        def _render_surface(wellfile: str) -> List[Dict[str, Any]]:
             """Update map"""
             wellname = Path(wellfile).stem
             wellfile = get_path(wellfile)
             surface = load_surface(get_path(self.surfacefiles[0]))
             well = load_well(wellfile)
             s_layer = make_surface_layer(
-                surface, name=self.surfacenames[0], hillshading=True,
+                surface,
+                name=self.surfacenames[0],
             )
             well_layer = make_well_layer(well, wellname)
-            return [s_layer, well_layer], "keep"
+            return [s_layer, well_layer]
 
-    def add_webvizstore(self):
+    def add_webvizstore(self) -> List[Tuple[Callable, list]]:
         return [
             *[(get_path, [{"path": fn}]) for fn in self.segyfiles],
             *[(get_path, [{"path": fn}]) for fn in self.surfacefiles],
@@ -320,5 +357,5 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
 
 
 @webvizstore
-def get_path(path) -> Path:
+def get_path(path: str) -> Path:
     return Path(path)
