@@ -1,18 +1,20 @@
+from typing import List, Tuple, Union, Dict, Callable, Any
 import io
 import os
 import json
 from uuid import uuid4
 from pathlib import Path
-from typing import List
 
-import numpy as np
 import xtgeo
+import numpy as np
+import pandas as pd
+import dash
 from dash.exceptions import PreventUpdate
 from dash.dependencies import Input, Output, State
 import dash_html_components as html
 import dash_core_components as dcc
 import webviz_core_components as wcc
-from webviz_subsurface_components import LayeredMap
+from webviz_subsurface_components import LeafletMap
 from webviz_config import WebvizPluginABC
 from webviz_config.webviz_store import webvizstore
 from webviz_config.common_cache import CACHE
@@ -77,18 +79,18 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
     # pylint: disable=too-many-arguments, too-many-locals
     def __init__(
         self,
-        app,
+        app: dash.Dash,
         ensembles: list,
         surfacefiles: list,
         surfacenames: list = None,
-        surfacefolder: Path = "share/results/maps",
+        surfacefolder: Path = Path("share/results/maps"),
         wellfiles: List[Path] = None,
         wellfolder: Path = None,
         wellsuffix: str = ".w",
         segyfiles: List[Path] = None,
         zonelog: str = None,
         marginal_logs: list = None,
-        zunit="depth (m)",
+        zunit: str = "depth (m)",
         zmin: float = None,
         zmax: float = None,
         zonemin: int = 1,
@@ -105,11 +107,11 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
             )
         self.wellfolder = wellfolder
         self.wellsuffix = wellsuffix
-        self.wellfiles = (
-            json.load(find_files(wellfolder, wellsuffix))
-            if wellfolder is not None
-            else [str(well) for well in wellfiles]
-        )
+        self.wellfiles: List[str]
+        if wellfolder is not None:
+            self.wellfiles = json.load(find_files(wellfolder, wellsuffix))
+        elif wellfiles is not None:
+            self.wellfiles = [str(well) for well in wellfiles]
 
         self.surfacefolder = surfacefolder
         self.surfacefiles = surfacefiles
@@ -128,7 +130,7 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
             for ens in ensembles
         }
 
-        self.realizations = get_realizations(
+        self.realizations: pd.DataFrame = get_realizations(
             ensemble_paths=self.ensembles, ensemble_set_name="EnsembleSet"
         )
         self.marginal_logs = marginal_logs
@@ -162,12 +164,12 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
         self.uid = uuid4()
         self.set_callbacks(app)
 
-    def ids(self, element):
+    def ids(self, element: str) -> str:
         """Generate unique id for dom element"""
         return f"{element}-id-{self.uid}"
 
     @property
-    def well_layout(self):
+    def well_layout(self) -> html.Div:
         return html.Div(
             children=html.Label(
                 children=[
@@ -180,13 +182,15 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                         ],
                         value=self.wellfiles[0],
                         clearable=False,
+                        persistence=True,
+                        persistence_type="session",
                     ),
                 ]
             )
         )
 
     @property
-    def surface_names_layout(self):
+    def surface_names_layout(self) -> html.Div:
         return html.Div(
             style=self.set_style(marginTop="20px"),
             children=html.Label(
@@ -200,13 +204,15 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                         value=self.surfacenames,
                         clearable=True,
                         multi=True,
+                        persistence=True,
+                        persistence_type="session",
                     ),
                 ]
             ),
         )
 
     @property
-    def ensemble_layout(self):
+    def ensemble_layout(self) -> html.Div:
         return html.Div(
             style=self.set_style(marginTop="20px"),
             children=html.Label(
@@ -221,13 +227,15 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                         value=list(self.ensembles.keys())[0],
                         clearable=False,
                         multi=False,
+                        persistence=True,
+                        persistence_type="session",
                     ),
                 ]
             ),
         )
 
     @property
-    def seismic_layout(self):
+    def seismic_layout(self) -> html.Div:
         if self.segyfiles:
             return html.Div(
                 style=self.set_style(marginTop="20px")
@@ -244,6 +252,8 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                             ],
                             value=self.segyfiles[0],
                             clearable=False,
+                            persistence=True,
+                            persistence_type="session",
                         ),
                     ]
                 ),
@@ -253,7 +263,7 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
         )
 
     @property
-    def marginal_log_layout(self):
+    def marginal_log_layout(self) -> html.Div:
         if self.marginal_logs is not None:
             return html.Div(
                 children=html.Label(
@@ -267,17 +277,19 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                             ],
                             placeholder="Display log",
                             clearable=True,
+                            persistence=True,
+                            persistence_type="session",
                         ),
                     ]
                 ),
             )
         return html.Div(
             style={"visibility": "hidden"},
-            children=dcc.Dropdown(id=self.ids("marginal-log")),
+            children=dcc.Dropdown(id=self.ids("marginal-log"), persistence=True),
         )
 
     @property
-    def intersection_option(self):
+    def intersection_option(self) -> html.Div:
         options = [
             {"label": "Keep zoom state", "value": "keep_zoom_state"},
             {"label": "Show surface fill", "value": "show_surface_fill"},
@@ -298,21 +310,33 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                     id=self.ids("sampling"),
                     debounce=True,
                     type="number",
+                    required=True,
                     value=self.sampling,
+                    persistence=True,
+                    persistence_type="session",
                 ),
                 html.Label("Extension"),
                 dcc.Input(
                     id=self.ids("nextend"),
                     debounce=True,
                     type="number",
+                    required=True,
                     value=self.nextend,
+                    persistence=True,
+                    persistence_type="session",
+                ),
+                dcc.Checklist(
+                    id=self.ids("options"),
+                    options=options,
+                    value=value,
+                    persistence=True,
+                    persistence_type="session",
                 ),
-                dcc.Checklist(id=self.ids("options"), options=options, value=value),
             ],
         )
 
     @property
-    def map_layout(self):
+    def map_layout(self) -> wcc.FlexBox:
         return wcc.FlexBox(
             children=[
                 dcc.Dropdown(
@@ -323,6 +347,8 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                     value=self.surfacenames[0],
                     clearable=False,
                     multi=False,
+                    persistence=True,
+                    persistence_type="session",
                 ),
                 dcc.Dropdown(
                     id=self.ids("surface-type"),
@@ -340,12 +366,14 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                     value="stddev",
                     clearable=False,
                     multi=False,
+                    persistence=True,
+                    persistence_type="session",
                 ),
             ],
         )
 
     @property
-    def layout(self):
+    def layout(self) -> wcc.FlexBox:
         return wcc.FlexBox(
             children=[
                 html.Div(
@@ -377,7 +405,16 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                             },
                             children=[
                                 self.map_layout,
-                                LayeredMap(height=400, id=self.ids("map"), layers=[]),
+                                LeafletMap(
+                                    id=self.ids("map"),
+                                    layers=[],
+                                    unitScale={},
+                                    autoScaleMap=True,
+                                    minZoom=-5,
+                                    updateMode="update",
+                                    mouseCoords={"position": "bottomright"},
+                                    colorBar={"position": "bottomleft"},
+                                ),
                             ],
                         ),
                         html.Button(
@@ -385,14 +422,16 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                             id=self.ids("show_map"),
                             children="Show map",
                         ),
-                        dcc.Store(id=self.ids("fencespec"), data=[]),
+                        dcc.Store(
+                            id=self.ids("fencespec"), storage_type="session", data=[]
+                        ),
                     ],
                 ),
             ],
         )
 
     @staticmethod
-    def set_style(columns=None, **kwargs):
+    def set_style(columns: str = None, **kwargs: Any) -> dict:
         if columns is not None:
             return {
                 "display": "grid",
@@ -403,7 +442,7 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
             }
         return {**kwargs}
 
-    def set_callbacks(self, app):
+    def set_callbacks(self, app: dash.Dash) -> None:
         @app.callback(
             [
                 Output(self.ids("graph"), "figure"),
@@ -421,20 +460,38 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
             ],
         )
         def _render_section(
-            surfacenames, ensemble, well, cube, options, sampling, nextend, marginal_log
-        ):
+            surfacenames: List[str],
+            ensemble: str,
+            wellfile: str,
+            segyfile: str,
+            options: List[str],
+            sampling: Union[int, float, None],
+            nextend: Union[int, float, None],
+            marginal_log: Union[str, None],
+        ) -> Tuple[dict, List[List[float]]]:
             """Update cross section"""
-            # Ensure list
-            surfacenames = (
-                surfacenames if isinstance(surfacenames, list) else [surfacenames]
-            )
+
+            # TODO(Sigurd) According to dcc.Dropdown doc, surfacename should always be of
+            # type List[str] since multi=True
+            if not isinstance(surfacenames, list):
+                raise TypeError("surfacenames must be of type list")
+
+            # TODO(Sigurd) Can we prohibit clearing of the sampling and nextend input
+            # fields (dcc.Input) in the client? Until we can, we must guard against sampling
+            # and nextend being None. This happens when the user clears the input field and we
+            # have not yet found a solution that prohibits the input field from being cleared.
+            # The situation can be slightly remedied by setting required=True which will highlight
+            # the missing value with a red rectangle.
+            if sampling is None or nextend is None:
+                raise PreventUpdate
+
             # Sort selected surfaces
             surfacenames = [name for name in self.surfacenames if name in surfacenames]
             surfacefiles = [
                 self.surfacefiles[self.surfacenames.index(name)]
                 for name in surfacenames
             ]
-            well = load_well(get_path(well))
+            well = load_well(get_path(wellfile))
             xsect = XSectionFigure(
                 well=well,
                 zmin=self.zmin,
@@ -449,14 +506,17 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
 
             for surfacename, surfacefile in zip(surfacenames, surfacefiles):
                 stat_surfs = get_surface_statistics(
-                    self.realizations, ensemble, surfacefile, self.surfacefolder
+                    self.realizations.to_dict(orient="records"),
+                    ensemble,
+                    surfacefile,
+                    self.surfacefolder,
                 )
                 xsect.plot_statistical_surface(
                     stat_surfs, name=surfacename, fill="show_surface_fill" in options
                 )
 
             if "show_seismic" in options:
-                cube = load_cube_data(get_path(cube))
+                cube = load_cube_data(get_path(segyfile))
                 xsect.plot_cube(cube)
 
             xsect.plot_well(
@@ -468,7 +528,9 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
             layout.update(self.plotly_theme["layout"])
             if "keep_zoom_state" in options:
                 layout["uirevision"] = "keep"
-            fencespec = [[coord[0], coord[1]] for coord in xsect.fence]
+            fencespec: List[List[float]] = [
+                [coord[0], coord[1]] for coord in xsect.fence
+            ]
             return {"data": xsect.data, "layout": layout}, fencespec
 
         @app.callback(
@@ -479,7 +541,7 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
             [Input(self.ids("show_map"), "n_clicks")],
             [State(self.ids("map_wrapper"), "style")],
         )
-        def _show_map(nclicks, style):
+        def _show_map(nclicks: int, style: dict) -> Tuple[dict, str]:
             """Show/hide map on button click"""
 
             if not nclicks:
@@ -491,7 +553,7 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
             return style, "Show map"
 
         @app.callback(
-            [Output(self.ids("map"), "layers"), Output(self.ids("map"), "uirevision")],
+            Output(self.ids("map"), "layers"),
             [
                 Input(self.ids("fencespec"), "data"),
                 Input(self.ids("surface-name"), "value"),
@@ -499,31 +561,43 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                 Input(self.ids("ensembles"), "value"),
             ],
         )
-        def _render_surface(fencespec, surfacename, surfacetype, ensemble):
+        def _render_surface(
+            fencespec: List[List[float]],
+            surfacename: str,
+            surfacetype: str,
+            ensemble: str,
+        ) -> List[dict]:
             """Update map"""
             intersect_layer = {
                 "name": "Well",
+                "id": "Well",
+                "action": "replace",
                 "checked": True,
                 "base_layer": False,
-                "data": [{"type": "polyline", "color": "red", "positions": fencespec,}],
+                "data": [
+                    {
+                        "type": "polyline",
+                        "color": "red",
+                        "positions": fencespec,
+                    }
+                ],
             }
 
             surface = get_surface_statistics(
-                self.realizations,
+                self.realizations.to_dict(orient="records"),
                 ensemble,
                 self.surfacefiles[self.surfacenames.index(surfacename)],
                 self.surfacefolder,
             )[surfacetype]
 
-            surface_layer = make_surface_layer(
-                surface, name=surfacename, hillshading=True
-            )
-            return [surface_layer, intersect_layer], "keep"
+            surface_layer = make_surface_layer(surface, name=surfacename)
+            return [surface_layer, intersect_layer]
 
-    def add_webvizstore(self):
-        store_functions = [
+    def add_webvizstore(self) -> List[Tuple[Callable, list]]:
+        store_functions: List[Tuple[Callable, list]] = [
             (get_path, [{"path": fn}]) for fn in self.segyfiles + self.wellfiles
         ]
+
         for ens in list(self.realizations["ENSEMBLE"].unique()):
             for surfacefile in self.surfacefiles:
                 store_functions.append(
@@ -531,7 +605,9 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
                         calculate_surface_statistics,
                         [
                             {
-                                "realdf": self.realizations,
+                                "realdf_dict": self.realizations.to_dict(
+                                    orient="records"
+                                ),
                                 "ensemble": ens,
                                 "surfacefile": surfacefile,
                                 "surfacefolder": self.surfacefolder,
@@ -560,8 +636,11 @@ e.g. [xtgeo](https://xtgeo.readthedocs.io/en/latest/).
 
 @webvizstore
 def calculate_surface_statistics(
-    realdf, ensemble, surfacefile, surfacefolder
+    realdf_dict: list, ensemble: str, surfacefile: str, surfacefolder: Path
 ) -> io.BytesIO:
+
+    realdf = pd.DataFrame(realdf_dict)
+
     fns = [
         os.path.join(real_path, surfacefolder, surfacefile)
         for real_path in list(realdf[realdf["ENSEMBLE"] == ensemble]["RUNPATH"])
@@ -582,16 +661,18 @@ def calculate_surface_statistics(
 
 
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
-def get_surface_statistics(realdf, ensemble, surfacefile, surfacefolder):
+def get_surface_statistics(
+    realdf_dict: list, ensemble: str, surfacefile: str, surfacefolder: Path
+) -> Dict[str, xtgeo.RegularSurface]:
     surfaces = json.load(
-        calculate_surface_statistics(realdf, ensemble, surfacefile, surfacefolder)
+        calculate_surface_statistics(realdf_dict, ensemble, surfacefile, surfacefolder)
     )
     return {
         statistic: surface_from_json(surface) for statistic, surface in surfaces.items()
     }
 
 
-def surface_to_json(surface):
+def surface_to_json(surface: xtgeo.RegularSurface) -> str:
     return json.dumps(
         {
             "ncol": surface.ncol,
@@ -606,7 +687,7 @@ def surface_to_json(surface):
     )
 
 
-def surface_from_json(surfaceobj):
+def surface_from_json(surfaceobj: Union[str, bytes]) -> xtgeo.RegularSurface:
     # See https://github.com/equinor/xtgeo/issues/405
     data = json.loads(surfaceobj)
     surface = xtgeo.RegularSurface(**data)
@@ -615,17 +696,17 @@ def surface_from_json(surfaceobj):
 
 
 @CACHE.memoize(timeout=CACHE.TIMEOUT)
-def get_surfaces(fns):
-    return xtgeo.surface.surfaces.Surfaces(fns)
+def get_surfaces(fns: List[str]) -> xtgeo.Surfaces:
+    return xtgeo.Surfaces(fns)
 
 
 @webvizstore
-def get_path(path) -> Path:
+def get_path(path: str) -> Path:
     return Path(path)
 
 
 @webvizstore
-def find_files(folder, suffix) -> io.BytesIO:
+def find_files(folder: Path, suffix: str) -> io.BytesIO:
     return io.BytesIO(
         json.dumps(
             sorted([str(filename) for filename in folder.glob(f"*{suffix}")])
